import unittest

from vivisect.symboliks.common import Var, Const, Arg, Constraint, Call, Mem, \
                                      o_add, o_sub, o_lshift, eq, ne, o_or, o_div, o_and, o_rshift
from vivisect.symboliks.effects import ReadMemory, WriteMemory, SetVariable, \
                                       CallFunction, ConstrainPath

from vivisect.symboliks.tests.helpers import MockEmulator
from vivisect.tests.helpers import MockVw

# Algebraic notations to test:
# idempotency, associativity, commutative, distributive, double negation, de morgans


def walker(path, cur, ctx):
    ctx.append(cur)

class TestSymbolikEffects(unittest.TestCase):
    def test_readmem(self):
        # idempotent
        addr = Var('eax', 4)
        size = Var('GetFileSize', 4)
        rm = ReadMemory(0x44, addr, size)
        self.assertEqual(str(rm), '[ eax : GetFileSize ]')
        rm.reduce()
        self.assertEqual(rm, ReadMemory(0x59, addr, size))

        # reduction
        addr = ((((Var('eax', 4) * Var('ebx', 4) + Const(9, 4)) - Const(4, 4))) << Const(3, 4)) * Const(2, 4)
        size = ((Const(0x73, 4) | Const(0x84, 4)) | Const(0xCC00, 4)) ^ Var('ecx', 4)
        rm = ReadMemory(0x44, addr, size)
        rm.reduce()
        self.assertEqual(str(rm), '[ (((eax * ebx) + 5) * 16) : (0x0000ccf7 ^ ecx) ]')
        self.assertEqual(str(rm.symaddr), '(((eax * ebx) + 5) * 16)')
        self.assertEqual(str(rm.symsize), '(0x0000ccf7 ^ ecx)')

        # walkTree
        addr = (Var('edx', 4) + Var('mm0', 4)) - Var('edx', 4)
        addr <<= (Var('foo', 4) + Var('foo', 4) - (Var('foo', 4) * Const(2, 4)))
        size = (Var('bar', 4) - Const(0x19, 4)) - (Var('biz', 4) - Const(0x20, 4))
        rm = ReadMemory(0x44, addr, size)
        visited = []
        rm.walkTree(walker, ctx=visited)
        answer = [
            'edx',
            'mm0',
            '(edx + mm0)',
            'edx',
            '((edx + mm0) - edx)',
            'foo',
            'foo',
            '(foo + foo)',
            'foo',
            '2',
            '(foo * 2)',
            '((foo + foo) - (foo * 2))',
            '(((edx + mm0) - edx) << ((foo + foo) - (foo * 2)))',
            'bar',
            '25',
            '(bar - 25)',
            'biz',
            '32',
            '(biz - 32)',
            '((bar - 25) - (biz - 32))',
        ]
        strd = [str(x) for x in visited]
        self.assertEqual(answer, strd)
        self.assertEqual(str(rm), '[ (((edx + mm0) - edx) << ((foo + foo) - (foo * 2))) : ((bar - 25) - (biz - 32)) ]')
        rm.reduce()
        self.assertEqual(str(rm), '[ mm0 : ((bar - biz) + 7) ]')

        # application
        emu = MockEmulator(MockVw())
        emu.setSymVariable('mm0', Var('bar', 4))
        emu.setSymVariable('biz', Const(4, 4))
        new_rm = rm.applyEffect(emu)
        self.assertEqual(str(new_rm), '[ bar : ((bar - 4) + 7) ]')
        new_rm.reduce()
        self.assertEqual(str(new_rm), '[ bar : (bar + 3) ]')

    def test_writemem(self):
        # reduction
        msg = 'the quick brown dog'
        base = Arg(0, 4) * Const(8, 4) | Const(0x410000, 4)
        size = Const(1, 4)

        writes = []
        for c in msg:
            valu = Const(ord(c), 4)
            wm = WriteMemory(0x44, base, size, valu)
            base += Const(1, 4)
            writes.append(wm)
        answer = [
            '[ ((arg0 * 8) | 0x00410000) : 1 ] = 116',
            '[ (((arg0 * 8) | 0x00410000) + 1) : 1 ] = 104',
            '[ (((arg0 * 8) | 0x00410000) + 2) : 1 ] = 101',
            '[ (((arg0 * 8) | 0x00410000) + 3) : 1 ] = 32',
            '[ (((arg0 * 8) | 0x00410000) + 4) : 1 ] = 113',
            '[ (((arg0 * 8) | 0x00410000) + 5) : 1 ] = 117',
            '[ (((arg0 * 8) | 0x00410000) + 6) : 1 ] = 105',
            '[ (((arg0 * 8) | 0x00410000) + 7) : 1 ] = 99',
            '[ (((arg0 * 8) | 0x00410000) + 8) : 1 ] = 107',
            '[ (((arg0 * 8) | 0x00410000) + 9) : 1 ] = 32',
            '[ (((arg0 * 8) | 0x00410000) + 10) : 1 ] = 98',
            '[ (((arg0 * 8) | 0x00410000) + 11) : 1 ] = 114',
            '[ (((arg0 * 8) | 0x00410000) + 12) : 1 ] = 111',
            '[ (((arg0 * 8) | 0x00410000) + 13) : 1 ] = 119',
            '[ (((arg0 * 8) | 0x00410000) + 14) : 1 ] = 110',
            '[ (((arg0 * 8) | 0x00410000) + 15) : 1 ] = 32',
            '[ (((arg0 * 8) | 0x00410000) + 16) : 1 ] = 100',
            '[ (((arg0 * 8) | 0x00410000) + 17) : 1 ] = 111',
            '[ (((arg0 * 8) | 0x00410000) + 18) : 1 ] = 103',
        ]

        for idx, write in enumerate(writes):
            write.reduce()
            self.assertEqual(str(write), answer[idx])

        # walkTree
        addr = Mem(Call(Var('seek', 4), Const(4, 4)), Const(16, 4))
        size = Var('ecx', 4)
        valu = Arg(0, 4)
        wm = WriteMemory(0x44, addr, size, valu)

        answer = [
            Arg(0, 4),
            Var('seek', 4),
            Call(Var('seek', 4), Const(4, 4)),
            Const(16, 4),
            Mem(Call(Var('seek', 4), Const(4, 4)), Const(16, 4)),
            Var('ecx', 4)
        ]
        visited = []
        wm.walkTree(walker, ctx=visited)
        self.assertEqual(answer, visited)

        # application
        emu = MockEmulator(MockVw())
        addr = Call(Var('malloc', 4), Const(4, 4), argsyms=[Var('eax', 4)])
        size = (Const(2, 4) ** Var('ebx', 4)) >> Const(2, 4)
        size *= Const(4, 4)
        valu = Mem(Const(0x41424344, 4), Const(8, 4))
        wm = WriteMemory(0x44, addr, size, valu)

        emu.setSymVariable('ebx', Const(6, 4))
        newwm = wm.applyEffect(emu)
        newwm.reduce(emu=emu)
        self.assertEqual(str(newwm), '[ malloc(eax) : 64 ] = mem[0x41424344:8]')

    def test_setvar(self):
        # reduction
        valu = Var('esp', 4) >> Var('ebp', 4)
        sv = SetVariable(0x44, 'vivisect', valu)
        self.assertEqual(str(sv), 'vivisect = (esp >> ebp)')

        valu = eq(Var('eax', 4), Var('eax', 4))
        sv = SetVariable(0x44, 'ultra', valu)
        self.assertEqual(str(sv), 'ultra = (eax == eax)')
        sv.reduce()
        self.assertEqual(str(sv), 'ultra = 1')

        # walkTree
        valu = ne(Const(0, 1), Const(0, 2))
        for i in range(8):
            valu |= Const(0x2 << i, 4)
        sv = SetVariable(0x44, 'vtrace', valu)
        visited = []
        sv.walkTree(walker, ctx=visited)
        answer = [
            Const(0, 1),
            Const(0, 2),
            ne(Const(0, 1), Const(0, 2)),
            Const(2, 4),
            o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4), 1),
            Const(4, 4),
            o_or(o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4), 1), Const(4, 4), 1),
            Const(8, 4),
            o_or(o_or(o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4), 1), Const(4, 4), 1), Const(8, 4), 1),
            Const(0x10, 4),
            o_or(o_or(o_or(o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4),1), Const(4, 4), 1), Const(8, 4), 1), Const(0x10, 4), 1),
            Const(0x20, 4),
            o_or(o_or(o_or(o_or(o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4), 1), Const(4, 4), 1), Const(8, 4), 1),Const(0x10, 4), 1), Const(0x20, 4), 1),
            Const(0x40, 4),
            o_or(o_or(o_or(o_or(o_or(o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4), 1),Const(4, 4), 1), Const(8, 4), 1), Const(0x10, 4), 1), Const(0x20, 4), 1), Const(0x40, 4), 1),
            Const(0x80, 4),
            o_or(o_or(o_or(o_or(o_or(o_or(o_or(ne(Const(0, 1), Const(0, 2)), Const(2, 4), 1),Const(4, 4), 1), Const(8, 4), 1), Const(0x10, 4), 1), Const(0x20, 4), 1), Const(0x40, 4), 1), Const(0x80, 4), 1),
            Const(0x100, 4),
            o_or(o_or(o_or(o_or(o_or(o_or(o_or(o_or(ne(Const(0, 1),Const(0, 2)),Const(2, 4), 1), Const(4, 4), 1), Const(8, 4), 1), Const(0x10, 4), 1), Const(0x20, 4), 1), Const(0x40, 4), 1),Const(0x80, 4), 1), Const(0x100, 4), 1)
        ]
        self.assertEqual(visited, answer)
        sv.reduce()
        self.assertEqual(sv, SetVariable(0x9001, 'vtrace', Const(254, 4)))

        # application
        emu = MockEmulator(MockVw())
        sv.applyEffect(emu)
        self.assertEqual(emu.getSymVariable('vtrace'), Const(254, 4))

    def test_callfunc(self):
        # reduction, with and without argsyms
        funcsym = Var('NeatoBurrito', 4)
        argsyms = [
            (Const(1, 4) ** Var('edx', 4)) ^ Const(0x10, 4),
            (Var('eax', 4) ^ Var('eax', 4) | Const(0x10, 4)) << Const(0, 4),
            (Var('ebx', 4) & Var('ebx', 4)) | Const(0x40, 4) | Const(0x80, 4),
            Mem(Const(0x41414141, 4), Const(32, 4)),
        ]
        cf = CallFunction(0x44, funcsym, argsyms=argsyms)
        self.assertEqual(str(cf), 'NeatoBurrito(((1 ** edx) ^ 16),(((eax ^ eax) | 16) << 0),(((ebx & ebx) | 64) | 128),mem[0x41414141:32])')
        cf.reduce()
        self.assertEqual(str(cf), 'NeatoBurrito(17,16,(ebx | 192),mem[0x41414141:32])')

        # walkTree
        funcsym = Var('NeatoBurrito', 4)
        argsyms = [
            (Var('eax', 4) + Const(47, 4)) + (Var('ebx', 4) + Const(12, 4)),
            Var('foo', 4) & Var('foo', 4),
            Var('eax', 4) / Const(4, 4),
            Var('eax', 4) >> Const(2, 4),
            Const(4, 4) / Var('eax', 4),
        ]
        cf = CallFunction(0x44, funcsym, argsyms=argsyms)
        answer = [
            Var('NeatoBurrito', 4),
            Var('eax', 4),
            Const(47, 4),
            o_add(Var("eax", 4), Const(47, 4), 4),
            Var('ebx', 4),
            Const(12, 4),
            o_add(Var('ebx', 4), Const(12, 4), 4),
            (Var('eax', 4) + Const(47, 4)) + (Var('ebx', 4) + Const(12, 4)),
            Var('foo', 4),
            Var('foo', 4),
            o_and(Var('foo', 4), Var('foo', 4), 4),
            Var('eax', 4),
            Const(4, 4),
            o_div(Var('eax', 4), Const(4, 4), 4),
            Var('eax', 4),
            Const(2, 4),
            o_rshift(Var('eax', 4), Const(2, 4), 4),
            Const(4, 4),
            Var('eax', 4),
            o_div(Const(4, 4), Var('eax', 4), 4),
        ]
        visited = []
        cf.walkTree(walker, ctx=visited)
        self.assertEqual(visited, answer)

        # application, with argsyms
        emu = MockEmulator(MockVw())
        emu.setSymVariable('eax', Const(100, 4))
        newcf = cf.applyEffect(emu=emu)
        newcf.reduce()
        self.assertEqual(str(newcf), 'NeatoBurrito((ebx + 159),foo,25,25,0)')

        # application, without argsyms
        # by itself, emu.applyFunctionCall is just a NOP
        emu = MockEmulator(MockVw())
        funcsym = Var('ebx', 4)
        cf = CallFunction(0x44, funcsym)
        newcf = cf.applyEffect(emu=emu)
        newcf.reduce()
        self.assertEqual(newcf, cf)

    def test_constrainpath(self):
        # reduction
        addrsym = Var('eax', 4) * Const(4, 4) + Const(0x4357, 4)
        consym = eq(Var('ebx', 4), Var('ebx', 4))
        cons = ConstrainPath(0x44, addrsym, consym)
        self.assertEqual(repr(cons), 'ConstrainPath( 0x00000044, o_add(o_mul(Var("eax", width=4),Const(0x00000004,4),4),Const(0x00004357,4),4), eq(Var("ebx", width=4),Var("ebx", width=4)) )')
        cons.reduce()
        self.assertEqual(repr(cons), 'ConstrainPath( 0x00000044, o_add(o_mul(Var("eax", width=4),Const(0x00000004,4),4),Const(0x00004357,4),4), Const(0x00000001,4) )')

        # walkTree
        addrsym = Var('eax', 4)
        consym = eq(eq(Var('ebx', 4), Var('edx', 4)), Const(1, 4))
        cons = ConstrainPath(0x44, addrsym, consym)
        visited = []

        cons.walkTree(walker, ctx=visited)
        answer = [
            Var('ebx', 4),
            Var('edx', 4),
            eq(Var('ebx', 4), Var('edx', 4)),
            Const(1, 4),
            eq(eq(Var('ebx', 4), Var('edx', 4)), Const(1, 4)),
        ]
        self.assertEqual(visited, answer)

        # application
        emu = MockEmulator(MockVw())
        addrsym = Var('ebx', 4)
        consym = eq(eq(Var('eax', 4), Var('ecx', 4)), Const(1, 4))
        cons = ConstrainPath(0x44, addrsym, consym)
        emu.setSymVariable('eax', Const(44, 4))
        emu.setSymVariable('ecx', Const(36, 4))
        newcons = cons.applyEffect(emu)
        newcons.reduce()
        self.assertEqual(newcons, ConstrainPath(0x1994578, Var('ebx', 4), Const(0, 4)))



import logging

import visgraph.pathcore as vg_path

import vivisect
import vivisect.exc as v_exc
import vivisect.impemu.monitor as viv_monitor
import vivisect.analysis.generic.codeblocks as viv_cb

import envi
import envi.common as e_cmn
import envi.archs.arm as e_arm

from envi.archs.arm.regs import *
from envi.archs.arm.const import *

from vivisect.const import *

logger = logging.getLogger(__name__)


class AnalysisMonitor(viv_monitor.AnalysisMonitor):

    def __init__(self, vw, fva, verbose=True):
        viv_monitor.AnalysisMonitor.__init__(self, vw, fva)
        self.retbytes = None
        self.badops = vw.arch.archGetBadOps()
        self.last_lr_pc = 0
        self.strictops = False
        self.returns = False
        self.infloops = []
        self.switchcases = 0

    def prehook(self, emu, op, starteip):

        try:
            tmode = emu.getFlag(PSR_T_bit)
            self.last_tmode = tmode
            if op in self.badops:
                emu.stopEmu()
                raise v_exc.BadOpBytes(op.va)

            viv_monitor.AnalysisMonitor.prehook(self, emu, op, starteip)

            loctup = emu.vw.getLocation(starteip)
            if loctup is None:
                # logger.debug("emulation: prehook: new LOC_OP  fva: 0x%x     starteip: 0x%x  flags: 0x%x", self.fva, starteip, op.iflags)
                arch = (envi.ARCH_ARMV7, envi.ARCH_THUMB)[(starteip & 1) | tmode]
                emu.vw.makeCode(starteip & -2, arch=arch)

            elif loctup[2] != LOC_OP:
                logger.info("ARG! emulation found opcode in an existing NON-OPCODE location  (0x%x):  0x%x: %s", loctup[0], op.va, op)
                emu.stopEmu()

            elif loctup[0] != starteip:
                logger.info("ARG! emulation found opcode in a location at the wrong address (0x%x):  0x%x: %s", loctup[0], op.va, op)
                emu.stopEmu()

            if op.iflags & envi.IF_RET:
                self.returns = True
                if len(op.opers):
                    if hasattr(op.opers, 'imm'):
                        self.retbytes = op.opers[0].imm

            # ARM gives us nice switchcase handling instructions
            # FIXME: wrap TB-handling into getBranches(emu) which is called by checkBranches during emulation
            if op.opcode in (INS_TBH, INS_TBB):
                if emu.vw.getVaSetRow('SwitchCases', op.va) is None:
                    base, tbl = analyzeTB(emu, op, starteip, self)
                    if None not in (base, tbl):
                        count = len(tbl)
                        self.switchcases += 1
                        emu.vw.setVaSetRow('SwitchCases', (op.va, op.va, count))

            elif op.opcode == INS_MOV:
                if len(op.opers) >= 2:
                    oper0 = op.opers[0]
                    oper1 = op.opers[1]

                    if isinstance(oper0, e_arm.ArmRegOper) and oper0.reg == REG_LR:
                        if isinstance(oper1, e_arm.ArmRegOper) and oper1.reg == REG_PC:
                            self.last_lr_pc = starteip

            elif op.opcode == INS_BX:
                if starteip - self.last_lr_pc <= 4:
                    # this is a call.  the compiler updated lr
                    logger.info("CALL by mov lr, pc; bx <foo> at 0x%x", starteip)
                    tgtva = op.opers[-1].getOperValue(op)
                    self.vw.makeFunction(tgtva)

            elif op.opcode == INS_ADD and op.opers[0].reg == REG_PC:
                # simple branch code
                if emu.vw.getVaSetRow('SwitchCases', op.va) is None:
                    base, tbl = analyzeADDPC(emu, op, starteip, self)
                    if None not in (base, tbl):
                        count = len(tbl)
                        self.switchcases += 1
                        emu.vw.setVaSetRow('SwitchCases', (op.va, op.va, count))

            elif op.opcode == INS_SUB and isinstance(op.opers[0], e_arm.ArmRegOper) and op.opers[0].reg == REG_PC:
                # simple branch code
                if emu.vw.getVaSetRow('SwitchCases', op.va) is None:
                    base, tbl = analyzeSUBPC(emu, op, starteip, self)
                    if None not in (base, tbl):
                        count = len(tbl)
                        self.switchcases += 1
                        emu.vw.setVaSetRow('SwitchCases', (op.va, op.va, count))

            if op.iflags & envi.IF_BRANCH:
                try:
                    tgt = op.getOperValue(0, emu)

                    if tgt == op.va:
                        logger.info("0x%x: +++++++++++++++ infinite loop +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++", op.va)
                        if op.va not in self.infloops:
                            self.infloops.append(op.va)
                            if 'InfiniteLoops' not in self.vw.getVaSetNames():
                                self.vw.addVaSet('InfiniteLoops', (('va', vivisect.VASET_ADDRESS, 'function', vivisect.VASET_STRING)))
                            self.vw.setVaSetRow('InfiniteLoops', (op.va, self.fva))

                except Exception as e:
                    self.logAnomaly(emu, self.fva, "0x%x: (%r) ERROR: %s" % (op.va, op, e))
                    logger.info("0x%x: (%r) ERROR: %s", op.va, op, e)

        except v_exc.BadOpBytes:
            raise

        except Exception as e:
            self.logAnomaly(emu, self.fva, "0x%x: (%r) ERROR: %s" % (op.va, op, e))
            logger.warning("0x%x: (%r)  ERROR: %s", op.va, op, e)

    def posthook(self, emu, op, starteip):
        if op.opcode == INS_BLX:
            emu.setFlag(PSR_T_bit, self.last_tmode)

        elif op.opcode == INS_MOVT:
            val = op.getOperValue(0, emu=emu)
            emu.vw.setSymHint(op.va, OP_SYMHINT_IDX, val)
            if emu.isValidPointer(val):
                emu.vw.addXref(op.va, val, vivisect.REF_PTR)

argnames = {
    0: ('r0', 0),
    1: ('r1', 1),
    2: ('r2', 2),
    3: ('r3', 3),
}


def archargname(idx):
    ret = argnames.get(idx)
    if ret is None:
        name = 'arg%d' % idx
    else:
        name, idx = ret
    return name


def buildFunctionApi(vw, fva, emu, emumon):
    argc = 0
    funcargs = []
    callconv = vw.getMeta('DefaultCall')
    undefregs = set(emu.getUninitRegUse())

    for argnum in range(len(argnames), 0, -1):
        argname, argid = argnames[argnum-1]
        if argid in undefregs:
            argc = argnum
            break

    if callconv == 'armcall':
        if emumon.stackmax > 0:
            targc = (emumon.stackmax >> 3) + 6
            if targc > 40:
                emumon.logAnomaly(emu, fva, 'Crazy Stack Offset Touched: 0x%.8x' % emumon.stackmax)
            else:
                argc = targc

        funcargs = [('int', archargname(i)) for i in range(argc)]

    api = ('int', None, callconv, None, funcargs)
    vw.setFunctionApi(fva, api)
    return api


def getAllReads(emu):
    '''
    WorkspaceEmulators optionally track paths through all code emulated.  While called a "path",
    this data is actually stored in a graph-like object where nodes are able to store properties.

    Read/Write logs are stored in these nodes.

    This function sorts through the emulator's Path structure and grab all Reads.  It uses
    getAllPaths() for the graph-like Path, which flattens out all paths and returns them as a 
    list of paths, each path being a list of codeblock nodes.

    The details are hidden with an accessor API, but the nodes are tuple of:
        (parent, [children], {properties})

    Yield generator
    Skips duplicates
    '''
    allaccess = set()
    for path in vg_path.getAllPaths(emu.path):
        for node in path:
            for read in vg_path.getNodeProp(node, 'readlog'):
                if read in allaccess:
                    # skip duplicate
                    continue

                allaccess.add(read)
                logger.log(e_cmn.SHITE, "getAllReads() => %r", repr(read))
                yield read


def getAllWrites(emu):
    '''
    WorkspaceEmulators optionally track paths through all code emulated.  While called a "path",
    this data is actually stored in a graph-like object where nodes are able to store properties.

    Read/Write logs are stored in these nodes.

    This function sorts through the emulator's Path structure and grab all Writes.  It uses
    getAllPaths() for the graph-like Path, which flattens out all paths and returns them as a 
    list of paths, each path being a list of codeblock nodes.

    The details are hidden with an accessor API, but the nodes are tuple of:
        (parent, [children], {properties})

    Yield generator
    Skips duplicates
    '''
    allaccess = set()
    for path in vg_path.getAllPaths(emu.path):
        for node in path:
            for write in vg_path.getNodeProp(node, 'writelog'):
                if write in allaccess:
                    # skip duplicate
                    continue

                allaccess.add(write)
                logger.log(e_cmn.SHITE, "getAllWrites() => %r", repr(write))
                yield write

def analyzeFunction(vw, fva):
    emu = vw.getEmulator(va=fva, logread=True, logwrite=True)
    emumon = AnalysisMonitor(vw, fva)
    emu.setEmulationMonitor(emumon)

    loc = vw.getLocation(fva)
    if loc is not None:
        lva, lsz, lt, lti = loc
        if lt == LOC_OP:
            if (lti & envi.ARCH_MASK) != envi.ARCH_ARMV7:
                emu.setFlag(PSR_T_bit, 1)
    else:
        logger.warning("NO LOCATION at FVA: 0x%x", fva)

    emu.runFunction(fva, maxhit=1)


    # find all xrefs that are read/written to
    xrefs = []
    for read in getAllReads(emu):
        tofrom = read[:2]
        if tofrom not in xrefs:
            xrefs.append(tofrom)

    for write in getAllWrites(emu):
        tofrom = write[:2]
        if tofrom not in xrefs:
            xrefs.append(tofrom)

    stackmap = emu.getMemoryMap(emu.getStackCounter())

    # make XREFs for the ones that are interesting/sane
    for x, y in xrefs:
        if emu.getVivTaint(y):
            continue

        if emu.getMemoryMap(y) == stackmap:
            continue

        vw.addXref(x, y, REF_PTR)

    # Do we already have API info in meta?
    # NOTE: do *not* use getFunctionApi here, it will make one!
    api = vw.getFunctionMeta(fva, 'api')
    if api is None:
        api = buildFunctionApi(vw, fva, emu, emumon)

    rettype,retname,callconv,callname,callargs = api

    argc = len(callargs)
    cc = emu.getCallingConvention(callconv)
    if cc is None:
        return

    stcount = cc.getNumStackArgs(emu, argc)
    stackidx = argc - stcount
    baseoff = cc.getStackArgOffset(emu, argc)

    # Register our stack args as function locals
    for i in range(stcount):

        vw.setFunctionLocal(fva, baseoff + ( i * 4 ), LSYM_FARG, i+stackidx)

    emumon.addAnalysisResults(vw, emu)

    # handle infinite loops (actually, while 1;)

    # switch-cases may have updated codeflow.  reanalyze
    viv_cb.analyzeFunction(vw, fva)
    # logger.debug("-- Arm EMU fmod: 0x%x" % fva)



# TODO: incorporate some of emucode's analysis but for function analysis... if it makes sense.


def analyzeTB(emu, op, starteip, amon):
    ######################### FIXME: ADD THIS TO getBranches(emu)
    ### DEBUGGING
    #raw_input("\n\n\nPRESS ENTER TO START TB: 0x%x" % op.va)
    logger.debug("\n\nTB at 0x%x", starteip)
    tsize = op.opers[0].tsize
    tbl = []
    basereg = op.opers[0].base_reg
    if basereg != REG_PC:
        base = emu.getRegister(basereg)
    else:
        base = op.opers[0].va

    logger.debug("\nbase: 0x%x", base)
    val0 = emu.readMemValue(base, tsize)

    if val0 > 0x100 + base:
        logger.debug("ummmm.. Houston we got a problem.  first option is a long ways beyond BASE")

    va = base
    while va < base + val0:
        nextoff = emu.readMemValue(va, tsize) * 2
        logger.debug("0x%x: -> 0x%x", va, nextoff + base)
        if nextoff == 0:
            logging.debug("Terminating TB at 0-offset")
            break

        if nextoff > 0x500:
            logging.debug("Terminating TB at LARGE - offset  (may be too restrictive): 0x%x", nextoff)
            break

        loc = emu.vw.getLocation(va)
        if loc is not None:
            logger.debug("Terminating TB at Location/Reference")
            logger.debug("%x, %d, %x, %r", loc)
            break

        tbl.append((va, nextoff))
        va += tsize

    logger.debug("%s: \n\t", op.mnem + '\n\t'.join(['0x%x (0x%x)' % (x, base + x) for v,x in tbl]))

    ###
    # for workspace emulation analysis, let's check the index register for sanity.
    idxreg = op.opers[0].offset_reg
    idx = emu.getRegister(idxreg)
    if idx > 0x40000000:
        emu.setRegister(idxreg, 0) # args handed in can be replaced with index 0

    jmptblbase = op.opers[0]._getOperBase(emu)
    jmptblval = emu.getOperAddr(op, 0)
    jmptbltgt = (emu.getOperValue(op, 0) * 2) + base
    logger.debug("0x%x: %r\njmptblbase: 0x%x\njmptblval:  0x%x\njmptbltgt:  0x%x", op.va, op, jmptblbase, jmptblval, jmptbltgt)
    #raw_input("PRESS ENTER TO CONTINUE")

    # make numbers and xrefs and names
    emu.vw.addXref(op.va, base, REF_DATA)
    emu.vw.makeName(op.va, 'br_tbl_%x' % op.va)

    case = 0
    for ova, nextoff in tbl:
        nexttgt = base + nextoff
        emu.vw.makeNumber(ova, 2)
        # check for loc first?
        if nexttgt & 1:
            nexttgt &= -2
            arch = envi.ARCH_THUMB
        else:
            arch = envi.ARCH_ARMV7

        emu.vw.makeCode(nexttgt, arch=arch)
        # check xrefs fist?
        emu.vw.addXref(op.va, nexttgt, REF_CODE)

        curname = emu.vw.getName(nexttgt)
        if curname is None:
            emu.vw.makeName(nexttgt, "case_%x_%x_%x" % (case, op.va, nexttgt))
        else:
            emu.vw.vprint("case_%x_%x_%x conflicts with existing name: %r" % (case, op.va, nexttgt, curname))
 
        case += 1

    return base, tbl

def analyzeADDPC(emu, op, starteip, emumon):
    count = None

    reg = op.opers[-1].reg
    cb = emu.vw.getCodeBlock(op.va)
    if cb is None:
        return None, None

    cbva, cbsz, cbfva = cb
    off = 0
    while off < cbsz:
        top = emu.vw.parseOpcode(cbva+off)
        if top.opcode == INS_CMP:
            # make sure this is a comparison for this register. 
            # the comparison should be the size of the switch-case
            for opidx in range(len(top.opers)):
                oper = top.opers[opidx]
                if isinstance(oper, e_arm.ArmRegOper):
                    if oper.reg != reg:
                        continue

                    #logger.debug("cmp op: ", top)
                    cntoidx = (1,0)[opidx]
                    cntoper = top.opers[cntoidx]
                    #logger.debug("cntoper: %d, %r  %r" % (cntoidx, cntoper, vars(cntoper)))
                    count = cntoper.getOperValue(top, emu)
                    #logger.debug("count = ", count)

        off += len(top)

    if not count or count is None or count > 10000:
        return None, None

    #logger.debug("Making ADDPC SwitchCase (count=%d):" % count)
    # wire up the switch-cases, name each one, etc...
    tbl = []
    for x in range(count):
        base = op.opers[-2].getOperValue(op, emu)
        base_reg = op.opers[-1].reg
        emu.setRegister(base_reg, x)
        idx = op.opers[-1].getOperValue(op, emu)
        nexttgt = base + idx
        #logger.debug("x=%x, base=%x, idx=%x (%x)  %r %r  %d" % (x,base,idx, nexttgt, op, op.opers, emu.getRegister(op.opers[-1].reg)))
        tbl.append((base+idx, x))
        emu.vw.makeCode(nexttgt)
        emu.vw.addXref(starteip, nexttgt, REF_CODE)

        curname = emu.vw.getName(nexttgt)
        if curname is None:
            emu.vw.makeName(nexttgt, "case_%x_%x_%x" % (x, starteip, nexttgt))
        else:
            emu.vw.vprint("case_%x_%x_%x conflicts with existing name: %r" % (x, starteip, nexttgt, curname))
 

    return base, tbl


def analyzeSUBPC(emu, op, starteip, emumon):
    count = None

    cb = emu.vw.getCodeBlock(op.va)
    if cb is None:
        return None, None

    off = 0
    reg = op.opers[1].reg
    cbva, cbsz, cbfva = cb
    while off < cbsz:
        top = emu.vw.parseOpcode(cbva+off)
        if top.opcode == INS_CMP:
            # make sure this is a comparison for this register. 
            # the comparison should be the size of the switch-case
            for opidx in range(len(top.opers)):
                oper = top.opers[opidx]
                if isinstance(oper, e_arm.ArmRegOper):
                    if oper.reg != reg:
                        continue

                    #logger.debug("cmp op: ", top)
                    cntoidx = (1,0)[opidx]
                    cntoper = top.opers[cntoidx]
                    #logger.debug("cntoper: %d, %r  %r" % (cntoidx, cntoper, vars(cntoper)))
                    count = cntoper.getOperValue(top, emu)
                    #logger.debug("count = ", count)

        off += len(top)

    if not count or count is None or count > 10000:
        return None, None

    #logger.debug("Making SUBPC SwitchCase (count=%d):" % count)
    # wire up the switch-cases, name each one, etc...
    tbl = []

    base = op.opers[-2].getOperValue(op, emu)
    base_reg = op.opers[1].reg

    for x in range(count):
        emu.setRegister(base_reg, x)
        idx = op.opers[-1].getOperValue(op, emu)
        nexttgt = base - idx
        #logger.debug("x=%x, base=%x, idx=%x (%x)  %r %r  %d" % (x,base,idx, nexttgt, op, op.opers, emu.getRegister(op.opers[-1].reg)))
        tbl.append((base+idx, x))
        emu.vw.makeCode(nexttgt)
        emu.vw.addXref(starteip, nexttgt, REF_CODE)

        curname = emu.vw.getName(nexttgt)
        if curname is None:
            emu.vw.makeName(nexttgt, "case_%x_%x_%x" % (x, starteip, nexttgt))
        else:
            emu.vw.vprint("case_%x_%x_%x conflicts with existing name: %r" % (x, starteip, nexttgt, curname))

    return base, tbl



'''
All the code related to vtrace process snapshots and TraceSnapshot classes.
'''
import copy
import pickle
import logging

import envi
import vtrace
import vtrace.platforms.base as v_base

logger = logging.getLogger(__name__)


class TraceSnapshot(vtrace.Trace, v_base.TracerBase):
    '''
    A trace snapshot is similar to a traditional "core file" except that
    you may also have memory only snapshots that are never written to disk.
    '''
    def __init__(self, snapdict):

        self.s_snapcache = {}
        self.s_snapdict = snapdict

        # a seperate parser for each version...
        if snapdict['version'] == 1:
            self.s_version = snapdict['version']
            self.s_threads = snapdict['threads']
            self.s_regs = snapdict['regs']
            self.s_maps = snapdict['maps']
            self.s_mem = snapdict['mem']
            self.metadata = snapdict['meta']
            self.s_stacktrace = snapdict['stacktrace']
            self.s_exe = snapdict['exe']
            self.s_fds = snapdict['fds']
            self.localvars = snapdict.get('vars', {})
        else:
            raise Exception('ERROR: Unknown snapshot version!')

        # In the ghetto!
        archname = self.metadata.get('Architecture')
        envi.stealArchMethods(self, archname)

        vtrace.Trace.__init__(self)
        v_base.TracerBase.__init__(self)
        # This will re-init meta... *sigh* set it back...
        self.metadata = snapdict['meta']

        # Steal the reg defs of the first thread
        rinfo = list(self.s_regs.items())[0][1]
        self.setRegisterInfo(rinfo)

        #FIXME hard-coded page size!
        self.s_map_lookup = {}
        for mmap in self.s_maps:
            for i in range(mmap[0], mmap[0] + mmap[1], 4096):
                self.s_map_lookup[i] = mmap

        # Lets get some symbol resolvers created for our libraries
        #for fname in self.getNormalizedLibNames():
            #subres = e_resolv.FileSymbol(fname, 

        self.running = False
        self.attached = True
        # So that we pickle
        self.bplock = None
        self.thread = None

    def saveToFd(self, fd):
        '''
        Save this snapshot to the given file like object
        for later reloading...
        '''
        pickle.dump(self.s_snapdict, fd)

    def saveToFile(self, filename):
        '''
        Save a snapshot to file for later reading in...
        '''
        with open(filename, 'wb') as f:
            self.saveToFd(f)

    def getMemoryMap(self, addr):
        if self.getMeta('Architecture') == 'amd64':
            base = addr & 0xfffffffffffff000
        else:
            base = addr & 0xfffff000

        return self.s_map_lookup.get(base, None)

    def platformGetFds(self):
        return self.s_fds

    def getExe(self):
        return self.s_exe

    def getStackTrace(self):
        tid = self.getMeta('ThreadId')
        tr = self.s_stacktrace.get(tid, None)
        if tr is None:
            raise Exception('ERROR: Invalid thread id specified')
        return tr

    def platformGetRegCtx(self, thrid):
        rinfo = self.s_regs.get(thrid)
        ctx = self.archGetRegCtx()
        ctx.setRegisterInfo(rinfo)
        return ctx

    def platformGetMaps(self):
        return self.s_maps

    def platformGetThreads(self):
        return self.s_threads

    def platformReadMemory(self, address, size):
        map = self.getMemoryMap(address)
        if map is None:
            raise Exception("ERROR: platformReadMemory says no map for 0x%.8x" % address)
        offset = address - map[0]  # Base address
        mapbytes = self.s_mem.get(map[0], None)
        if mapbytes is None:
            raise vtrace.PlatformException("ERROR: Memory map at 0x%.8x is not backed!" % map[0])
        if len(mapbytes) == 0:
            raise vtrace.PlatformException("ERROR: Memory Map at 0x%.8x is backed by ''" % map[0])

        ret = mapbytes[offset:offset+size]
        rlen = len(ret)
        # We may have a cross-map read, just recurse for the rest
        if rlen != size:
            ret += self.platformReadMemory(address+rlen, size-rlen)
        return ret

    def platformWriteMemory(self, address, bytes):
        map = self.getMemoryMap(address)
        if map is None:
            raise Exception("ERROR: platformWriteMemory says no map for 0x%.8x" % address)
        offset = address - map[0]
        mapbytes = self.s_mem[map[0]]
        self.s_mem[map[0]] = mapbytes[:offset] + bytes + mapbytes[offset+len(bytes):]

    def platformDetach(self):
        pass

    def platformParseBinary(self, *args):
        logger.warning('FIXME FAKE PLATFORM PARSE BINARY: %s', args)

    # Over-ride register *caching* subsystem to store/retrieve
    # register information in pure dictionaries
    def cacheRegs(self, threadid):
        pass

    # FIXME regs in snapshots are broke...

    def syncRegs(self):
        pass


def loadSnapshot(filename):
    '''
    Load a vtrace process snapshot from a file
    '''
    with open(filename, 'rb') as f:
        return TraceSnapshot(pickle.load(f))


def takeSnapshot(trace):
    '''
    Take a snapshot of the process from the current state and return
    a reference to a tracer which wraps a "snapshot" or "core file".
    '''
    sd = dict()
    orig_thread = trace.getMeta("ThreadId")

    regs = dict()
    stacktrace = dict()

    for thrid, tdata in trace.getThreads().items():
        ctx = trace.getRegisterContext(thrid)
        reginfo = ctx.getRegisterInfo()
        regs[thrid] = reginfo
        try:
            stacktrace[thrid] = trace.getStackTrace()
        except Exception as e:
            logger.warning("Failed to get stack trace for thread 0x%.8x (%s)", thrid, e)

    mem = dict()
    maps = []
    for base, size, perms, fname in trace.getMemoryMaps():
        try:
            mem[base] = trace.readMemory(base, size)
            maps.append((base, size, perms, fname))
        except Exception as msg:
            logger.warning("Can't snapshot memmap at 0x%.8x (%s)", base, msg)

    # if we're on windows, grab and store TEBs as metadata
    plat = trace.getMeta('Platform')
    if plat == 'windows':
        trace.setMeta('TEBs', dict(trace.win32threads))

    # If the contents here change, change the version...
    sd['version'] = 1
    sd['threads'] = trace.getThreads()
    sd['regs'] = regs
    sd['maps'] = maps
    sd['mem'] = mem
    sd['meta'] = copy.deepcopy(trace.metadata)
    sd['stacktrace'] = stacktrace
    sd['exe'] = trace.getExe()
    sd['fds'] = trace.getFds()
    sd['vars'] = trace.localvars

    return TraceSnapshot(snapdict=sd)



import vtrace.tests as vt_tests

breakpoints = {
    'windows': 'ntdll.NtTerminateProcess',
    'linux': 'libc.exit',
    'freebsd': 'libc.exit',
}


class VtraceExpressionTest(vt_tests.VtraceProcessTest):
    def test_vtrace_sym(self):
        plat = self.trace.getMeta('Platform')
        symname = breakpoints.get(plat)
        entry = self.trace.parseExpression(symname)
        addEntry = self.trace.parseExpression(symname + " + 5")
        self.assertTrue(entry + 5 == addEntry)

    def test_baselib(self):
        plat = self.trace.getMeta('Platform')
        libname = breakpoints.get(plat).split('.')[0]
        entry = self.trace.parseExpression(libname)
        addEntry = self.trace.parseExpression(libname + " + 5")
        # grab a symbol in the library and compare offsets against that?
        self.assertTrue(entry + 5 == addEntry)




"""
The guts for the i386 envi opcode disassembler.
"""

import struct

import envi
import envi.bits as e_bits

# Grab our register enums etc...
from envi.const import *
from .regs import *

from . import opconst
from . import opcode86
all_tables = opcode86.tables86

# Our instruction prefix masks
# NOTE: table 3-4 (section 3.6) of intel 1 shows how REX/OP_SIZE
# interact...
INSTR_PREFIX = 0x0001
PREFIX_LOCK = 0x0002
PREFIX_REPNZ = 0x0004
PREFIX_REPZ = 0x0008
PREFIX_REP = 0x0010
PREFIX_REP_SIMD = 0x0020
PREFIX_REP_MASK = PREFIX_REPNZ | PREFIX_REPZ | PREFIX_REP | PREFIX_REP_SIMD
PREFIX_OP_SIZE = 0x0040
PREFIX_ADDR_SIZE = 0x0080
PREFIX_SIMD = 0x0100
PREFIX_CS = 0x0200
PREFIX_SS = 0x0400
PREFIX_DS = 0x0800
PREFIX_ES = 0x1000
PREFIX_FS = 0x2000
PREFIX_GS = 0x4000
PREFIX_REG_MASK = 0x8000

# So these prefixes aren't mandatory in the strict sense that all instructions have to have them
# but from section 2.1.2 of the intel manual which states:
# Two-byte opcode formats for general-purpose and SIMD instructions consist of one of the following:
# * An escape opcode byte 0FH as the primary opcode and a second opcode byte.
# * A mandatory prefix (66H, F2H, or F3H), an escape opcode byte, and a second opcode byte (same as previous
# bullet).

MANDATORY_PREFIXES = [False for i in range(0xFF)]
MANDATORY_PREFIXES[0xF2] = True
MANDATORY_PREFIXES[0xF3] = True
MANDATORY_PREFIXES[0x66] = True


# envi.registers meta offsets
RMETA_LOW8 = 0x00080000
RMETA_HIGH8 = 0x08080000
RMETA_LOW16 = 0x00100000
RMETA_LOW128 = 0x00800000

# Use a list here instead of a dict for speed (max 255 anyway)
i386_prefixes = [ None for i in range(256) ]
i386_prefixes[0xF0] = PREFIX_LOCK
i386_prefixes[0xF2] = PREFIX_REPNZ
i386_prefixes[0xF3] = PREFIX_REP
i386_prefixes[0x2E] = PREFIX_CS
i386_prefixes[0x36] = PREFIX_SS
i386_prefixes[0x3E] = PREFIX_DS
i386_prefixes[0x26] = PREFIX_ES
i386_prefixes[0x64] = PREFIX_FS
i386_prefixes[0x65] = PREFIX_GS
i386_prefixes[0x66] = PREFIX_OP_SIZE
i386_prefixes[0x67] = PREFIX_ADDR_SIZE

# The scale byte index into this for multiplier imm
scale_lookup = (1, 2, 4, 8)

# A set of instructions that are considered privileged (mark with IF_PRIV)
# FIXME this should be part of the opcdode tables!
priv_lookup = {
    "int": True,
    "in": True,
    "out": True,
    "insb": True,
    "outsb": True,
    "insd": True,
    "outsd": True,
    "vmcall": True,
    "vmlaunch": True,
    "vmresume": True,
    "vmxoff": True,
    "vmread": True,
    "vmwrite": True,
    "rsm": True,
    "lar": True,
    "lsl": True,
    "clts": True,
    "invd": True,
    "wbinvd": True,
    "wrmsr": True,
    "rdmsr": True,
    "sysexit": True,
    "lgdt": True,
    "lidt": True,
    "lmsw": True,
    "monitor": True,
    "mwait": True,
    "vmclear": True,
    "vmptrld": True,
    "vmptrst": True,
    "vmxon": True,
}

# Map of codes to their respective envi flags
iflag_lookup = {
    opconst.INS_RET: envi.IF_NOFALL | envi.IF_RET,
    opconst.INS_CALL: envi.IF_CALL,
    opconst.INS_HALT: envi.IF_NOFALL,
    opconst.INS_DEBUG: envi.IF_NOFALL,
    opconst.INS_CALLCC: envi.IF_CALL | envi.IF_COND,
    opconst.INS_BRANCH: envi.IF_NOFALL | envi.IF_BRANCH,
    opconst.INS_BRANCHCC: envi.IF_BRANCH | envi.IF_COND,
    opconst.INS_MOVCC: envi.IF_COND,
    opconst.INS_XCHGCC: envi.IF_COND,
    opconst.INS_TRET: envi.IF_NOFALL | envi.IF_RET,
    opconst.INS_HALT: envi.IF_NOFALL,
    # opconst.INS_TRAP: envi.IF_NOFALL,
    opconst.INS_TRAPCC: envi.IF_NOFALL | envi.IF_COND,
    opconst.INS_DEBUG: envi.IF_NOFALL,
    opconst.INS_INVALIDOP: envi.IF_NOFALL,
    opconst.INS_OFLOW: envi.IF_NOFALL,
}

PLATMODS = {
    'linux': 0x80,
    'windows': 0x2e
}

sizenames = ["" for x in range(65)]
sizenames[1] = "byte"
sizenames[2] = "word"
sizenames[4] = "dword"
sizenames[8] = "qword"
sizenames[16] = "oword"    # xmm regs, can also be dqword
sizenames[32] = "yword"    # ymm regs, can also be qqword
sizenames[64] = "zword"    # zmm regs, can also be dqqword

operand_range = (2, 3, 4)

MODE_16 = 0
MODE_32 = 1
MODE_64 = 2

# used in coinjunction with the MODE_* values above
MODESIZE = [
    2,
    4,
    8,
]

addr16_modes = [
    (REG_BX, REG_SI),
    (REG_BX, REG_DI),

    (REG_BP, REG_SI),
    (REG_BP, REG_DI),

    (REG_SI, ),
    (REG_DI, ),

    (),

    (REG_BX, ),
]

def addrToName(mcanv, va):
    sym = mcanv.syms.getSymByAddr(va)
    if sym is not None:
        return repr(sym)
    return "0x%.8x" % va

###########################################################################
#
# Operand objects for the i386 architecture
#
class i386RegOper(envi.RegisterOper):

    def __init__(self, reg, tsize):
        self.reg = reg
        self.tsize = tsize

    def repr(self, op):
        return self._dis_regctx.getRegisterName(self.reg)

    def getOperValue(self, op, emu=None):
        if emu is None:
            return None  # This operand type requires an emulator
        return emu.getRegister(self.reg)

    def getOperAddr(self, op, emu=None):
        return None

    def setOperValue(self, op, emu, value):
        emu.setRegister(self.reg, value)

    def render(self, mcanv, op, idx):
        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            # FIXME: bug?  what should this be?
            mcanv.addNameText(hint, typename="registers")
        else:
            name = self._dis_regctx.getRegisterName(self.reg)
            rname = self._dis_regctx.getRegisterName(self.reg & RMETA_NMASK)
            mcanv.addNameText(name, name=rname, typename="registers")

    def __eq__(self, other):
        if not isinstance(other, i386RegOper):
            return False
        if other.reg != self.reg:
            return False
        if other.tsize != self.tsize:
            return False
        return True

class i386ImmOper(envi.ImmedOper):
    """
    An operand representing an immediate.
    """
    def __init__(self, imm, tsize):
        self.imm = imm
        self.tsize = tsize

    def repr(self, op):
        ival = self.imm
        if self.tsize == 6:
            return "0x%.4x:0x%.8x" % (ival>>32, ival&0xffffffff)
        if ival > 4096:
            return "0x%.8x" % ival
        return str(ival)

    def getOperValue(self, op, emu=None):
        return self.imm

    def render(self, mcanv, op, idx):
        value = self.imm
        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            if mcanv.mem.isValidPointer(value):
                mcanv.addVaText(hint, value)
            else:
                mcanv.addNameText(hint)
        elif mcanv.mem.isValidPointer(value):
            name = addrToName(mcanv, value)
            mcanv.addVaText(name, value)
        else:

            if self.tsize == 6:
                mcanv.addNameText("0x%.4x:0x%.8x" % (value>>32, value&0xffffffff))
            elif self.imm >= 4096:
                mcanv.addNameText('0x%.8x' % value)
            else:
                mcanv.addNameText(str(value))

    def __eq__(self, other):
        if not isinstance(other, i386ImmOper):
            return False
        if other.imm != self.imm:
            return False
        if other.tsize != self.tsize:
            return False
        return True

class i386PcRelOper(envi.Operand):
    """
    This is the operand used for EIP relative offsets
    for operands on instructions like jmp/call
    """
    def __init__(self, imm, tsize):
        self.imm = imm
        self.tsize = tsize

    def repr(self, op):
        return "0x%.8x" % (op.va + op.size + self.imm)

    def isImmed(self):
        return True

    def isDiscrete(self):
        return True # Based on op.va...

    def getOperValue(self, op, emu=None):
        return op.va + op.size + self.imm

    def getOperAddr(self, op, emu=None):
        return None

    def render(self, mcanv, op, idx):
        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            mcanv.addVaText(hint, value)
        else:
            value = op.va + op.size + self.imm
            name = addrToName(mcanv, value)
            mcanv.addVaText(name, value)

    def __eq__(self, other):
        if not isinstance(other, i386PcRelOper):
            return False
        if other.imm != self.imm:
            return False
        if other.tsize != self.tsize:
            return False
        return True

class i386RegMemOper(envi.DerefOper):
    """
    An operand which represents the result of reading/writing memory from the
    dereference (with possible displacement) from a given register.
    """
    def __init__(self, reg, tsize, disp=0):
        self.reg = reg
        self.tsize = tsize
        self.disp = disp
        self._is_deref = True

    def repr(self, op):
        r = self._dis_regctx.getRegisterName(self.reg)
        if self.disp > 0:
            return "%s [%s + %d]" % (sizenames[self.tsize], r, self.disp)
        elif self.disp < 0:
            return "%s [%s - %d]" % (sizenames[self.tsize], r, abs(self.disp))
        return "%s [%s]" % (sizenames[self.tsize], r)

    def getOperValue(self, op, emu=None):
        # This operand type requires an emulator
        if emu is None:
            return None
        return emu.readMemValue(self.getOperAddr(op, emu), self.tsize)

    def setOperValue(self, op, emu, val):
        emu.writeMemValue(self.getOperAddr(op, emu), val, self.tsize)

    def getOperAddr(self, op, emu=None):
        if emu is None:
            return None  # This operand type requires an emulator
        base, size = emu.getSegmentInfo(op)
        rval = emu.getRegister(self.reg)
        return e_bits.unsigned(base + rval + self.disp, emu.imem_psize)

    def isDeref(self):
        # The disassembler may reach in and set this (if lea...)
        return self._is_deref

    def render(self, mcanv, op, idx):
        mcanv.addNameText(sizenames[self.tsize])
        mcanv.addText(" [")
        name = self._dis_regctx.getRegisterName(self.reg)
        rname = self._dis_regctx.getRegisterName(self.reg & RMETA_NMASK)
        mcanv.addNameText(name, name=rname, typename="registers")
        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            mcanv.addText(" + ")
            mcanv.addNameText(hint)

        else:
            if mcanv.mem.isValidPointer(self.disp):
                mcanv.addText(" + ")
                name = addrToName(mcanv, self.disp)
                mcanv.addVaText(name, self.disp)
            elif self.disp > 0:
                mcanv.addText(" + ")
                mcanv.addNameText(str(self.disp))
            elif self.disp < 0:
                mcanv.addText(" - ")
                mcanv.addNameText(str(abs(self.disp)))
        mcanv.addText("]")

    def __eq__(self, other):
        if not isinstance(other, i386RegMemOper):
            return False
        if other.reg != self.reg:
            return False
        if other.disp != self.disp:
            return False
        if other.tsize != self.tsize:
            return False
        return True

class i386ImmMemOper(envi.DerefOper):
    """
    An operand which represents the dereference (memory read/write) of
    a memory location associated with an immediate.
    """
    def __init__(self, imm, tsize):
        self.imm = imm
        self.tsize = tsize
        self._is_deref = True

    def isDeref(self):
        # The disassembler may reach in and set this (if lea...)
        return self._is_deref

    def isDiscrete(self):
        return True

    def repr(self, op):
        return "%s [0x%.8x]" % (sizenames[self.tsize], self.imm)

    def getOperValue(self, op, emu=None):
        if emu is None: return None # This operand type requires an emulator
        return emu.readMemValue(self.getOperAddr(op, emu), self.tsize)

    def setOperValue(self, op, emu, val):
        emu.writeMemValue(self.getOperAddr(op, emu), val, self.tsize)

    def getOperAddr(self, op, emu=None):
        ret = self.imm
        if emu is not None:
            base, size = emu.getSegmentInfo(op)
            ret = e_bits.unsigned(ret + base, emu.imem_psize)
        return ret

    def render(self, mcanv, op, idx):
        mcanv.addNameText(sizenames[self.tsize])
        mcanv.addText(" [")
        value = self.imm

        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            mcanv.addVaText(hint, value)
        else:
            name = addrToName(mcanv, value)
            mcanv.addVaText(name, value)

        mcanv.addText("]")

    def __eq__(self, other):
        if not isinstance(other, i386ImmMemOper):
            return False
        if other.imm != self.imm:
            return False
        if other.tsize != self.tsize:
            return False
        return True

class i386SibOper(envi.DerefOper):
    """
    An operand which represents the result of reading/writting memory from the
    dereference (with possible displacement) from a given register.
    """
    def __init__(self, tsize, reg=None, imm=None, index=None, scale=1, disp=0):
        self.reg = reg
        self.imm = imm
        self.index = index
        self.scale = scale
        self.tsize = tsize
        self.disp = disp
        self._is_deref = True

    def __eq__(self, other):
        if not isinstance(other, i386SibOper):
            return False
        if other.imm != self.imm:
            return False
        if other.reg != self.reg:
            return False
        if other.index != self.index:
            return False
        if other.scale != self.scale:
            return False
        if other.disp != self.disp:
            return False
        if other.tsize != self.tsize:
            return False
        return True

    def isDeref(self):
        return self._is_deref

    def repr(self, op):

        r = "%s [" % sizenames[self.tsize]

        if self.reg is not None:
            r += self._dis_regctx.getRegisterName(self.reg)

        if self.imm is not None:
            r += "0x%.8x" % self.imm

        if self.index is not None:
            r += " + %s" % self._dis_regctx.getRegisterName(self.index)
            if self.scale != 1:
                r += " * %d" % self.scale

        if self.disp > 0:
            r += " + %d" % self.disp
        elif self.disp < 0:
            r += " - %d" % abs(self.disp)

        r += "]"

        return r

    def getOperValue(self, op, emu=None):
        if emu is None: return None # This operand type requires an emulator
        return emu.readMemValue(self.getOperAddr(op, emu), self.tsize)

    def setOperValue(self, op, emu, val):
        emu.writeMemValue(self.getOperAddr(op, emu), val, self.tsize)

    def getOperAddr(self, op, emu=None):
        if emu is None: return None # This operand type requires an emulator

        ret = 0

        if self.imm is not None:
            ret += self.imm

        if self.reg is not None:
            ret += emu.getRegister(self.reg)

        if self.index is not None:
            ret += (emu.getRegister(self.index) * self.scale)

        if emu.imem_psize == 4:
            ret &= 0xFFFFFFFF
        elif emu.imem_psize == 8:
            ret &= 0xFFFFFFFFFFFFFFFF

        # Handle x86 segmentation
        base, size = emu.getSegmentInfo(op)
        ret += base

        return e_bits.unsigned(ret + self.disp, emu.imem_psize)

    def _getOperBase(self, emu=None):
        # Special SIB only method for getting the SIB base value
        if self.imm:
            return self.imm
        if emu:
            return emu.getRegister(self.reg)
        return None

    def render(self, mcanv, op, idx):

        mcanv.addNameText(sizenames[self.tsize])
        mcanv.addText(" [")
        if self.imm is not None:
            name = addrToName(mcanv, self.imm)
            mcanv.addVaText(name, self.imm)

        if self.reg is not None:
            name = self._dis_regctx.getRegisterName(self.reg)
            rname = self._dis_regctx.getRegisterName(self.reg&RMETA_NMASK)
            mcanv.addNameText(name, name=rname, typename="registers")

        # Does our SIB have a scale
        if self.index is not None:
            mcanv.addText(" + ")
            name = self._dis_regctx.getRegisterName(self.index)
            rname = self._dis_regctx.getRegisterName(self.index&RMETA_NMASK)
            mcanv.addNameText(name, name=rname, typename="registers")
            if self.scale != 1:
                mcanv.addText(" * ")
                mcanv.addNameText(str(self.scale))

        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            mcanv.addText(" + ")
            mcanv.addNameText(hint)

        else:
            # If we have a displacement, add it.
            if self.disp != 0:
                mcanv.addText(" + ")
                mcanv.addNameText(str(self.disp))

        mcanv.addText("]")

class i386Opcode(envi.Opcode):

    # Printable prefix names
    prefix_names = [
        #(PREFIX_ADDR_SIZE, "addr"),
        (PREFIX_LOCK, "lock"),
        (PREFIX_REPNZ, "repnz"),
        (PREFIX_REP, "rep"),
        (PREFIX_CS, "cs"),
        (PREFIX_SS, "ss"),
        (PREFIX_DS, "ds"),
        (PREFIX_ES, "es"),
        (PREFIX_FS, "fs"),
        (PREFIX_GS, "gs"),
    ]

    def getBranches(self, emu=None):
        ret = []

        # we're done here.
        if self.opcode == opconst.INS_HALT:
            return []

        # To start with we have no flags ( except our arch )
        flags = self.iflags & envi.ARCH_MASK
        addb = False

        # If we are a conditional branch, even our fallthrough
        # case is conditional...
        if self.opcode == opconst.INS_BRANCHCC:
            flags |= envi.BR_COND
            addb = True

        # If we can fall through, reflect that...
        if not self.iflags & envi.IF_NOFALL:
            ret.append((self.va + self.size, flags | envi.BR_FALL))

        # In intel, if we have no operands, it has no
        # further branches...
        if len(self.opers) == 0:
            return ret

        # Check for a call...
        if self.opcode == opconst.INS_CALL:
            flags |= envi.BR_PROC
            addb = True

        # A conditional call?  really?  what compiler did you use? ;)
        elif self.opcode == opconst.INS_CALLCC:
            flags |= (envi.BR_PROC | envi.BR_COND)
            addb = True

        elif self.opcode == opconst.INS_BRANCH:
            oper0 = self.opers[0]
            if isinstance(oper0, i386SibOper) and oper0.scale == 4:
                # In the case with no emulator, note that our deref is
                # from the base of a table. If we have one, parse out all the
                # valid pointers from our base
                base = oper0._getOperBase(emu)
                if emu is None:
                    ret.append((base, flags | envi.BR_DEREF | envi.BR_TABLE))

                else:
                    # Since we're parsing this out, lets just resolve the derefs
                    # for our caller...
                    dest = emu.readMemValue(base, oper0.tsize)
                    while emu.isValidPointer(dest):
                        ret.append((dest, envi.BR_COND))
                        base += oper0.tsize
                        dest = emu.readMemValue(base, oper0.tsize)
            else:
                addb = True

        if addb:
            oper0 = self.opers[0]
            if oper0.isDeref():
                flags |= envi.BR_DEREF
                tova = oper0.getOperAddr(self, emu=emu)
            else:
                tova = oper0.getOperValue(self, emu=emu)

            ret.append((tova, flags))

        return ret

    def render(self, mcanv):
        """
        Render this opcode to the specified memory canvas
        """
        if self.prefixes:
            pfx = self.getPrefixName()
            if pfx:
                mcanv.addNameText("%s: " % pfx, pfx)

        mcanv.addNameText(self.mnem, typename="mnemonic")
        mcanv.addText(" ")

        # Allow each of our operands to render
        imax = len(self.opers)
        lasti = imax - 1
        for i in range(imax):
            oper = self.opers[i]
            oper.render(mcanv, self, i)
            if i != lasti:
                mcanv.addText(",")


class i386Disasm:

    def __init__(self, mode=MODE_32):
        self._dis_mode = mode
        self._dis_prefixes = i386_prefixes
        self._dis_regctx = i386RegisterContext()
        self._dis_oparch = envi.ARCH_I386
        self._dis_default_size = MODESIZE[mode]
        self.ptrsize = 4

        # This will make function lookups nice and quick
        self._dis_amethods = [ None for x in range(1 + (opconst.ADDRMETH_LAST>>16)) ]
        self._dis_amethods[opconst.ADDRMETH_A>>16] = self.ameth_a
        self._dis_amethods[opconst.ADDRMETH_C>>16] = self.ameth_c
        self._dis_amethods[opconst.ADDRMETH_D>>16] = self.ameth_d
        self._dis_amethods[opconst.ADDRMETH_E>>16] = self.ameth_e
        self._dis_amethods[opconst.ADDRMETH_M>>16] = self.ameth_e
        self._dis_amethods[opconst.ADDRMETH_N>>16] = self.ameth_n
        self._dis_amethods[opconst.ADDRMETH_Q>>16] = self.ameth_q
        self._dis_amethods[opconst.ADDRMETH_R>>16] = self.ameth_e
        self._dis_amethods[opconst.ADDRMETH_W>>16] = self.ameth_w
        self._dis_amethods[opconst.ADDRMETH_I>>16] = self.ameth_i
        self._dis_amethods[opconst.ADDRMETH_J>>16] = self.ameth_j
        self._dis_amethods[opconst.ADDRMETH_O>>16] = self.ameth_o
        self._dis_amethods[opconst.ADDRMETH_G>>16] = self.ameth_g
        self._dis_amethods[opconst.ADDRMETH_P>>16] = self.ameth_p
        self._dis_amethods[opconst.ADDRMETH_S>>16] = self.ameth_s
        self._dis_amethods[opconst.ADDRMETH_U>>16] = self.ameth_u
        self._dis_amethods[opconst.ADDRMETH_V>>16] = self.ameth_v
        self._dis_amethods[opconst.ADDRMETH_X>>16] = self.ameth_x
        self._dis_amethods[opconst.ADDRMETH_Y>>16] = self.ameth_y

        # Offsets used to add in addressing method parsers
        # MMX is just a meta reg of st
        self.ROFFSETSIMD  = getRegOffset(i386regs, "xmm0")
        self.ROFFSETDEBUG = getRegOffset(i386regs, "debug0")
        self.ROFFSETCTRL  = getRegOffset(i386regs, "ctrl0")
        self.ROFFSETTEST  = getRegOffset(i386regs, "test0")
        self.ROFFSETSEG   = getRegOffset(i386regs, "es")
        self.ROFFSETFPU   = getRegOffset(i386regs, "st0")
        # Note: getRegOffset doesn't work on meta registers and mm* are aliases of the
        # st registers, so we use getRegisterIndex instead
        self.ROFFSETMMX   = self._dis_regctx.getRegisterIndex("mm0")

    def parse_modrm(self, byte, prefixes=0):
        # Pass in a string with an offset for speed rather than a new string
        mod = (byte >> 6) & 0x3
        reg = (byte >> 3) & 0x7
        rm = byte & 0x7
        return (mod, reg, rm)

    def byteRegOffset(self, val, prefixes=0):
        # NOTE: This is used for high byte metas in 32 bit mode only
        if val < 4:
            return val + RMETA_LOW8
        return (val-4) + RMETA_HIGH8

    def shortend_parse_modrm(self, bytez, offset, opersize, regbase=0, prefixes=0):
        '''
        This is to handle the 16 version of oper addressing. ~1/2 of the table is
        completely different from the 32 bit version
        '''
        mod, reg, rm = self.parse_modrm(bytez[offset])
        size = 1
        opers = addr16_modes[rm]
        # SIB oper for the 16 bit addressing stuff because it actually fits
        if mod == 0:
            if len(opers) == 2:
                return (size, i386SibOper(opersize, reg=opers[0], imm=None, index=opers[1], scale=1))
            elif len(opers) == 1:
                return (size, i386RegMemOper(opers[0], opersize))
            else:
                imm = e_bits.parsebytes(bytez, offset + size, 2)
                size += 2
                return (size, i386ImmMemOper(imm, opersize))

        elif mod == 1:
            imm = e_bits.parsebytes(bytez, offset + size, 1)
            size += 1
            if len(opers) == 2:
                return (size, i386SibOper(opersize, reg=opers[0], disp=imm, index=opers[1], scale=1))
            elif len(opers) == 1:
                return (size, i386RegMemOper(opers[0], opersize, disp=imm))
            else:
                return i386ImmMemOper(imm, opersize)
        elif mod == 2:
            imm = e_bits.parsebytes(bytez, offset + size, 2)
            size += 2
            if len(opers) == 2:
                return i386SibOper(opersize, reg=opers[0], disp=imm, index=opers[1], scale=1)
            elif len(opers) == 1:
                return (size, i386RegMemOper(opers[0], opersize, disp=imm))
            else:
                return (size, i386ImmMemOper(imm, opersize))
        elif mod == 3:
            if opersize == 1:
                rm = self.byteRegOffset(rm, prefixes=prefixes)
            elif opersize == 2:
                rm += RMETA_LOW16
            return (size, i386RegOper(rm+regbase, opersize))
        else:
            raise Exception("(16bit) How does mod == %d" % mod)

    # Parse modrm as though addr mode might not be just a reg, but only for 32bit mode
    def extended_parse_modrm(self, bytez, offset, opersize, regbase=0, prefixes=0):
        """
        Return a tuple of (size, Operand)
        """
        if prefixes & PREFIX_ADDR_SIZE:
            if opersize == 4 and self.ptrsize == 4:
                return self.shortend_parse_modrm(bytez, offset, opersize, regbase=regbase, prefixes=prefixes)

        mod, reg, rm = self.parse_modrm(bytez[offset])

        size = 1

        if mod == 3: # Easy one, just a reg
            # FIXME only use self.byteRegOffset in 32 bit mode, NOT 64 bit...
            if opersize == 1:
                rm = self.byteRegOffset(rm, prefixes=prefixes)
            elif opersize == 2:
                rm += RMETA_LOW16
            return (size, i386RegOper(rm+regbase, opersize))

        elif mod == 0:
            # means we are [reg] unless rm == 4 (SIB) or rm == 5 ([imm32])
            # BUT JOKES -- the table is totally different in 16 bit mode BECAUSE WHY
            if rm == 5:
                imm = e_bits.parsebytes(bytez, offset + size, 4)
                size += 4
                # NOTE: in 64 bit mode, *this* is where we differ, (This case is RIP relative)
                return(size, i386ImmMemOper(imm, opersize))

            elif rm == 4:
                sibsize, scale, index, base, imm = self.parse_sib(bytez, offset+size, mod, prefixes=prefixes)
                size += sibsize
                if base is not None:
                    base += regbase    # Adjust for different register addressing modes
                if index is not None:
                    index += regbase    # Adjust for different register addressing modes
                oper = i386SibOper(opersize, reg=base, imm=imm, index=index, scale=scale_lookup[scale])
                return (size, oper)

            else:
                return(size, i386RegMemOper(regbase+rm, opersize))

        elif mod == 1:
            # mod 1 means we are [ reg + disp8 ] (unless rm == 4 which means sib + disp8)
            if rm == 4:
                sibsize, scale, index, base, imm = self.parse_sib(bytez, offset+size, mod, prefixes=prefixes)
                size += sibsize
                disp = e_bits.parsebytes(bytez, offset+size, 1, sign=True)
                size += 1
                if base is not None:
                    base += regbase    # Adjust for different register addressing modes
                if index is not None:
                    index += regbase    # Adjust for different register addressing modes
                oper = i386SibOper(opersize, reg=base, index=index, scale=scale_lookup[scale], disp=disp)
                return (size,oper)
            else:
                x = e_bits.signed(bytez[offset+size], 1)
                size += 1
                return(size, i386RegMemOper(regbase+rm, opersize, disp=x))

        elif mod == 2:
            # Means we are [ reg + disp32 ] (unless rm == 4  which means SIB + disp32)
            if rm == 4:
                sibsize, scale, index, base, imm = self.parse_sib(bytez,offset+size,mod, prefixes=prefixes)
                size += sibsize
                disp = e_bits.parsebytes(bytez, offset + size, 4, sign=True)
                size += 4
                if base is not None:
                    base += regbase    # Adjust for different register addressing modes
                if index is not None:
                    index += regbase    # Adjust for different register addressing modes
                oper = i386SibOper(opersize, reg=base, imm=imm, index=index, scale=scale_lookup[scale], disp=disp)
                return (size, oper)

            else:
                # NOTE: Immediate displacements in SIB are still 4 bytes in 64 bit mode
                disp = e_bits.parsebytes(bytez, offset+size, 4, sign=True)
                size += 4
                return(size, i386RegMemOper(regbase+rm, opersize, disp=disp))

        else:
            raise Exception("How does mod == %d" % mod)

    def parse_sib(self, bytez, offset, mod, prefixes=0):
        """
        Return a tuple of (size, scale, index, base, imm)
        """
        byte = bytez[offset]
        scale = (byte >> 6) & 0x3
        index = (byte >> 3) & 0x7
        base  = byte & 0x7
        imm = None

        size = 1

        # Special SIB case with no index reg
        if index == 4:
            index = None

        # Special SIB case with possible immediate
        if base == 5:
            if mod == 0: # [ imm32 + index * scale ]
                base = None
                imm = e_bits.parsebytes(bytez, offset+size, 4, sign=False)
                size += 4
            # FIXME is there special stuff needed here?
            elif mod == 1:
                pass
                #raise "OMG MOD 1"
            elif mod == 2:
                pass
                #raise "OMG MOD 2"

        return (size, scale, index, base, imm)


    def _dis_calc_tsize(self, opertype, prefixes, operflags):
        """
        Use the oper type and prefixes to decide on the tsize for
        the operand.
        """
        mode = MODE_32

        sizelist = opcode86.OPERSIZE.get(opertype, None)
        if sizelist is None:
            raise Exception("OPERSIZE FAIL: %.8x" % opertype)

        if prefixes & PREFIX_OP_SIZE:
            mode = MODE_16

        return sizelist[mode]

    def disasm(self, bytez, offset, va, extra=None):
        # Stuff for opcode parsing
        tabdesc = all_tables[0] # A tuple (optable, shiftbits, mask byte, sub, max)
        startoff = offset # Use startoff as a size knob if needed

        # Stuff we'll be putting in the opcode object
        optype = None # This gets set if we successfully decode below
        mnem = None
        operands = []

        all_prefixes = 0
        prefix_len = 0
        last_pref = 0

        while True:

            obyte = bytez[offset]

            # This line changes in 64 bit mode
            p = self._dis_prefixes[obyte]
            if p is None:
                break
            all_prefixes |= p
            last_pref = obyte
            offset += 1
            continue

        # At this point we should have all the possible prefixes, but some may be mandatory ones that we
        # need to not use as display prefixes and use as jumps in the table instead.
        # So we're going to lie to the rest of the code in order to use them as we want
        obyte = bytez[offset]
        ppref = [(None, None)]
        if obyte == 0x0f and MANDATORY_PREFIXES[last_pref]:
            obyte = last_pref
            ppref.append((last_pref, i386_prefixes[last_pref]))

        #pdone = False
        decodings = []
        mainbyte = offset
        prefixes = all_prefixes

        # as noted above, since we can have prefixes that may or may not be mandatory,
        # we roll through those and pop off the last one, since there's two cases we have
        # to deal with: a normal prefix that just modifies the opers, and a mandatory prefix
        # that modifies the instruction semantics entirely. Either way, the mandatory prefix
        # takes precedence and whichever one wins will be at the end of the list <decodings>
        for pref, onehot in ppref:
            if pref is not None:
                obyte = pref
                offset = mainbyte
                prefixes = all_prefixes & (~onehot)
            else:
                offset = mainbyte
                obyte = bytez[offset]

            tabdesc = all_tables[0]
            while True:
                if (obyte > tabdesc[4]):
                    tabdesc = all_tables[tabdesc[5]]

                tabidx = ((obyte - tabdesc[3]) >> tabdesc[1]) & tabdesc[2]
                if tabidx > len(tabdesc[0]):
                    break
                opdesc = tabdesc[0][tabidx]

                # Hunt down multi-byte opcodes
                nexttable = opdesc[0]
                if nexttable != 0: # If we have a sub-table specified, use it.
                    tabdesc = all_tables[nexttable]

                    offset += 1
                    obyte = bytez[offset]

                    continue

                # We are now on the final table...
                mnem = opdesc[6]
                optype = opdesc[1]
                if tabdesc[2] == 0xff:
                    offset += 1  # For our final opcode byte
                break
            if optype != 0:
                decodings.append((opdesc, offset, prefixes))

        if not len(decodings):
            raise envi.InvalidInstruction(bytez=bytez[startoff:startoff+16], va=va)

        opdesc, offset, all_prefixes = decodings.pop()
        optype = opdesc[1]
        mnem = opdesc[6]

        if optype == 0:
            raise envi.InvalidInstruction(bytez=bytez[startoff:startoff+16], va=va)

        operoffset = 0
        # Begin parsing operands based off address method
        for i in operand_range:

            oper = None # Set this if we end up with an operand
            osize = 0

            # Pull out the operand description from the table
            operflags = opdesc[i]
            opertype = operflags & opcode86.OPTYPE_MASK
            addrmeth = operflags & opcode86.ADDRMETH_MASK

            # If there are no more operands, break out of the loop!
            if operflags == 0:
                break

            tsize = self._dis_calc_tsize(opertype, all_prefixes, operflags)

            # If addrmeth is zero,we have operands embedded in the opcode
            if addrmeth == 0:
                osize = 0
                oper = self.ameth_0(operflags, opdesc[5+i], tsize, all_prefixes)
            else:
                ameth = self._dis_amethods[addrmeth >> 16]
                if ameth is None:
                    raise Exception("Implement Addressing Method 0x%.8x" % addrmeth)

                # NOTE: Depending on your addrmethod you may get beginning of operands, or offset
                try:
                    if addrmeth == opcode86.ADDRMETH_I or addrmeth == opcode86.ADDRMETH_J:
                        osize, oper = ameth(bytez, offset+operoffset, tsize, all_prefixes, operflags)

                        # If we are a sign extended immediate and not the same as the other operand,
                        # do the sign extension during disassembly so nothing else has to worry about it..
                        if operflags & opcode86.OP_SIGNED:
                            if len(operands) and tsize != operands[-1].tsize:
                                otsize = operands[-1].tsize
                                oper.imm = e_bits.sign_extend(oper.imm, oper.tsize, otsize)
                                oper.tsize = otsize
                            elif not len(operands):
                                oper.imm = e_bits.sign_extend(oper.imm, oper.tsize, self._dis_default_size)
                                oper.tsize = self._dis_default_size

                    else:
                        osize, oper = ameth(bytez, offset, tsize, all_prefixes, operflags)
                        # so in the opcode maps intel directly mentions that some opcodes are
                        # ADDRMETH_W but if the operand is a memory ref, it's always of a specific
                        # size, with no rhyme or reason as to which it is. So we directly embed
                        # that knowledge into the opcodes mappings we maintain and pluck it out
                        # here.
                        if oper and oper.isDeref():
                            memsz = opconst.OP_EXTRA_MEMSIZES[(operflags & opconst.OP_MEMMASK) >> 4]
                            if memsz is not None:
                                oper.tsize = memsz

                except struct.error as e:
                    # Catch struct unpack errors due to insufficient data length
                    raise envi.InvalidInstruction(bytez=bytez[startoff:startoff+16])

            if oper is not None:
                # This is a filty hack for now...
                oper._dis_regctx = self._dis_regctx
                operands.append(oper)

            operoffset += osize

        # Pull in the envi generic instruction flags
        iflags = iflag_lookup.get(optype & 0xFFFF, 0) | self._dis_oparch

        if all_prefixes & PREFIX_REP_MASK:
            iflags |= envi.IF_REPEAT

        if priv_lookup.get(mnem, False):
            iflags |= envi.IF_PRIV

        # Lea will have a reg-mem/sib operand with _is_deref True, but should be false
        if optype == opcode86.INS_LEA:
            operands[1]._is_deref = False

        ret = i386Opcode(va, optype, mnem, all_prefixes, (offset-startoff)+operoffset, operands, iflags)

        if ret.opcode == opconst.INS_TRAP and extra:
            # if we're on linux-i386, this is how they do syscalls, otherwise, it's a trap
            plat = extra.get('platform')
            if plat:
                if ret.getOperValue(0) != PLATMODS.get(plat, None):
                    ret.iflags |= envi.IF_NOFALL

        return ret

    # Declare all the address method parsers here!

    def ameth_0(self, operflags, operval, tsize, prefixes):
        # Special address method for opcodes with embedded operands
        if operflags & opcode86.OP_REG:
            if prefixes & PREFIX_OP_SIZE and operval & RMETA_NMASK == operval:
                operval |= RMETA_LOW16
            width = self._dis_regctx.getRegisterWidth(operval) >> 3
            return i386RegOper(operval, width)
        elif operflags & opcode86.OP_IMM:
            return i386ImmOper(operval, tsize)
        raise Exception("Unknown ameth_0! operflags: 0x%.8x" % operflags)

    def ameth_a(self, bytez, offset, tsize, prefixes, operflags):
        imm = e_bits.parsebytes(bytez, offset, tsize)
        # seg = e_bits.parsebytes(bytez, offset+tsize, 2)
        # THIS BEING GHETTORIGGED ONLY EFFECTS callf jmpf - unghettorigged by atlas
        # FIXME: envi.intel.ameth_a skipping seg prefix %d" % seg)
        return (tsize, i386ImmOper(imm, tsize))

    def ameth_e(self, bytez, offset, tsize, prefixes, operflags):
        return self.extended_parse_modrm(bytez, offset, tsize, prefixes=prefixes)

    def ameth_n(self, bytez, offset, tsize, prefixes, operflags):
        mod,reg,rm = self.parse_modrm(bytez[offset])
        return (1, i386RegOper(rm + self.ROFFSETMMX, tsize))

    def ameth_q(self, bytez, offset, tsize, prefixes, operflags):
        mod,reg,rm = self.parse_modrm(bytez[offset])
        if mod == 3:
            return (1, i386RegOper(rm + self.ROFFSETMMX, tsize))
        return self.extended_parse_modrm(bytez, offset, tsize, prefixes=prefixes)

    def ameth_w(self, bytez, offset, tsize, prefixes, operflags):
        mod,reg,rm = self.parse_modrm(bytez[offset])
        if mod == 3:
            return (1, i386RegOper(rm + self.ROFFSETSIMD, tsize))
        return self.extended_parse_modrm(bytez, offset, tsize, prefixes=prefixes)

    def ameth_i(self, bytez, offset, tsize, prefixes, operflags):
        imm = e_bits.parsebytes(bytez, offset, tsize)
        return (tsize, i386ImmOper(imm, tsize))

    def ameth_j(self, bytez, offset, tsize, prefixes, operflags):
        imm = e_bits.parsebytes(bytez, offset, tsize, sign=True)
        return (tsize, i386PcRelOper(imm, tsize))

    def ameth_o(self, bytez, offset, tsize, prefixes, operflags):
        # NOTE: displacement *stays* 32 bit even with REX
        # (but 16 bit should probably be supported)
        imm = e_bits.parsebytes(bytez, offset, self.ptrsize, sign=False)
        return (self.ptrsize, i386ImmMemOper(imm, tsize))

    def ameth_g(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        if tsize == 1: reg = self.byteRegOffset(reg, prefixes)
        elif tsize == 2: reg += RMETA_LOW16
        return (0, i386RegOper(reg, tsize))

    def ameth_c(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        return (0, i386RegOper(reg+self.ROFFSETCTRL, tsize))

    def ameth_d(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        return (0, i386RegOper(reg+self.ROFFSETDEBUG, tsize))

    def ameth_p(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        return (0, i386RegOper(reg+self.ROFFSETMMX, tsize))

    def ameth_s(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        return (0, i386RegOper(reg+self.ROFFSETSEG, tsize))

    def ameth_u(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        return (1, i386RegOper(rm+self.ROFFSETSIMD, tsize))

    def ameth_v(self, bytez, offset, tsize, prefixes, operflags):
        mod, reg, rm = self.parse_modrm(bytez[offset])
        return (0, i386RegOper(reg+self.ROFFSETSIMD, tsize))

    def ameth_x(self, bytez, offset, tsize, prefixes, operflags):
        #FIXME this needs the DS over-ride, but is only for outsb which we don't support
        return (0, i386RegMemOper(REG_ESI, tsize))

    def ameth_y(self, bytez, offset, tsize, prefixes, operflags):
        #FIXME this needs the ES over-ride, but is only for insb which we don't support
        return (0, i386RegMemOper(REG_ESI, tsize))

if __name__ == '__main__':
    import envi.archs
    envi.archs.dismain(i386Disasm())



from PyQt5 import QtCore
from PyQt5.QtWidgets import QTreeView


class VQTreeItem(object):

    def __init__(self, rowdata, parent):
        self.parent = parent
        self.rowdata = list(rowdata)
        self.children = []

    def append(self, rowdata):
        child = VQTreeItem(rowdata, self)
        self.children.append(child)
        return child

    def delete(self, rowdata):
        idx = 0
        for child in self.children:
            if child.rowdata == rowdata:
                return self.children.pop(idx)

            idx += 1

    def child(self, row):
        return self.children[row]

    def childCount(self):
        return len(self.children)

    def columnCount(self):
        return len(self.rowdata)

    def data(self, column):
        if column > len(self.rowdata):
            return None
        return str(self.rowdata[column])

    def row(self):
        '''
        Retrieve our row number.
        '''
        if self.parent:
            return self.parent.children.index(self)
        return 0

class VQTreeModel(QtCore.QAbstractItemModel):
    '''
    A QT tree model that uses the tree API from visgraph
    to hold the data...
    '''

    columns = ('A first column!', 'The Second Column!')
    editable = None
    dragable = False

    def __init__(self, parent=None, columns=None):

        if columns is not None:
            self.columns = columns

        QtCore.QAbstractItemModel.__init__(self, parent=parent)
        self.rootnode = VQTreeItem((), None)

        if self.editable is None:
            self.editable = [False,] * len(self.columns)

    def vqEdited(self, pnode, col, value):
        return value

    def append(self, rowdata, parent=None):
        if parent is None:
            parent = self.rootnode

        pidx = self.createIndex(parent.row(), 0, parent)
        i = len(parent.children)
        self.beginInsertRows(pidx, i, i)
        node = parent.append(rowdata)
        self.endInsertRows()
        self.layoutChanged.emit()
        return node

    def vqDelRow(self, rowdata, parent=None):
        if parent is None:
            parent = self.rootnode

        parent.delete(rowdata)

    def sort(self, colnum, order=0):
        self.layoutAboutToBeChanged.emit()
        self.rootnode.children.sort(key=lambda k: k.rowdata[colnum], reverse=bool(order))
        self.layoutChanged.emit()

    def flags(self, index):
        if not index.isValid():
            return 0
        flags = QtCore.QAbstractItemModel.flags(self, index)
        col = index.column()
        if self.editable[col]:
            flags |= QtCore.Qt.ItemIsEditable
        if self.dragable:
            flags |= QtCore.Qt.ItemIsDragEnabled
        return flags

    def columnCount(self, parent=None):
        return len(self.columns)

    def data(self, index, role):
        if not index.isValid():
            return None

        item = index.internalPointer()
        if role == QtCore.Qt.DisplayRole:
            return item.data(index.column())

        if role == QtCore.Qt.UserRole:
            return item

        return None

    def setData(self, index, value, role=QtCore.Qt.EditRole):

        node = index.internalPointer()
        if not node:
            return False

        # If this is the edit role, fire the vqEdited thing
        if role == QtCore.Qt.EditRole:
            value = self.vqEdited(node, index.column(), value)
            if value is None:
                return False

        node.rowdata[index.column()] = value
        self.dataChanged.emit(index, index)

        return True

    def headerData(self, column, orientation, role):
        if ( orientation == QtCore.Qt.Horizontal and
             role == QtCore.Qt.DisplayRole):

            return self.columns[column]

        return None

    def index(self, row, column, parent):

        if not self.hasIndex(row, column, parent):
            return QtCore.QModelIndex()

        pitem = parent.internalPointer()
        if not pitem:
            pitem = self.rootnode

        item = pitem.child(row)
        if not item:
            return QtCore.QModelIndex()

        return self.createIndex(row, column, item)

    def parent(self, index):

        if not index.isValid():
            return QtCore.QModelIndex()

        item = index.internalPointer()
        if not item:
            return QtCore.QModelIndex()

        pitem = item.parent

        if pitem == self.rootnode:
            return QtCore.QModelIndex()

        if pitem is None:
            return QtCore.QModelIndex()

        return self.createIndex(pitem.row(), 0, pitem)

    def rowCount(self, parent=QtCore.QModelIndex()):

        if parent.column() > 0:
            return 0

        pitem = parent.internalPointer()
        if not pitem:
            pitem = self.rootnode

        return len(pitem.children)

class VQTreeView(QTreeView):

    def __init__(self, parent=None, cols=None, **kwargs):
        QTreeView.__init__(self, parent=parent, **kwargs)
        self.setSortingEnabled(True)
        self.setAlternatingRowColors(True)

        if cols is not None:
            model = VQTreeModel(parent=self, columns=cols)
            self.setModel(model)

    def vqSizeColumns(self):
        c = self.model().columnCount()
        for i in range(c):
            self.resizeColumnToContents(i)

    def setModel(self, model):
        model.dataChanged.connect( self.dataChanged )
        model.rowsInserted.connect( self.rowsInserted )
        return QTreeView.setModel(self, model)



import io
import zipfile
import unittest

import vivisect
import vivisect.tests.helpers as helpers


class MalwareTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        testpath = helpers.getTestPath('windows', 'i386', 'malware.zip')
        cls.embed_vw = vivisect.VivWorkspace()
        cls.mmap_vw = vivisect.VivWorkspace()
        with zipfile.ZipFile(testpath, mode='r') as zp:
            fd = io.BytesIO(zp.open('mal_memmaps.exe', pwd=b'infected', mode='r').read())
            # DEV: intentionally skipping analysis for now since the tests I have in mind
            # initially don't need it (and it adds a lot of time that isn't worth it until we
            # add a lot more tests)
            cls.mmap_vw.loadFromFd(fd)

            # DEV: This one needs analysis, but luckily only takes ~2 seconds to analyze
            fd = io.BytesIO(zp.open('mal_carve.exe', pwd=b'infected', mode='r').read())
            cls.embed_vw.loadFromFd(fd)
            cls.embed_vw.analyze()

    def test_memorymaps_basicperms(self):
        vw = self.mmap_vw
        maps = set([
            (268435456, 0x1000, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
            (268439552, 0x66c00, 5, '0e25aa791c9119108af073bc9e9d0fa2'),
            (268861440, 0xdc00, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
            (268918784, 0x13600, 6, '0e25aa791c9119108af073bc9e9d0fa2'),
            (269004800, 0x4600, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
        ])
        self.assertEqual(maps, set(vw.getMemoryMaps()))

    def test_locations(self):
        # make sure none of the locations in any of the other maps are opcodes
        vw = self.mmap_vw
        for lva, lsize, ltype, linfo in vw.getLocations():
            memmap = vw.getMemoryMap(lva)
            if memmap[0] != 0x10001000:
                self.assertNotEqual(ltype, 5)

    def test_carve_pe(self):
        # all embedded PEs should be marked as dead data
        vw = self.embed_vw
        self.assertTrue(vw.isDeadData(0x10004030))
        self.assertTrue(vw.isDeadData(0x10018074))
        self.assertTrue(vw.isDeadData(0x1001e030))



'''
A utility for creating "remote applications" which are dcode
enabled and cobra driven.  All API arguments/returns *must* be
serializable using msgpack.

NOTE: enabling a dcode server means source for local python modules
      will be delivered directly to clients over the network!

Running a remote application will also attempt to prefer code from
the server rather than the local python current working directory.
( and uses multiprocessing for import/process isolation )
'''
import os
import sys
import importlib
import subprocess
import multiprocessing

import cobra
import cobra.dcode


class RemoteAppServer:
    def __init__(self):
        pass


def shareRemoteApp(name, appsrv=None, daemon=None, port=443):
    '''
    Fire an appropriate dcode enabled cobra daemon and share
    the appsrv object with the given name.
    '''
    if appsrv is None:
        appsrv = RemoteAppServer()

    if daemon is None:
        daemon = cobra.CobraDaemon(msgpack=True, port=port)
        daemon.fireThread()

    cobra.dcode.enableDcodeServer(daemon=daemon)
    return daemon.shareObject(appsrv, name)


def getAndRunApp(uri):
    # We dont want our *local* code, we want the remote code.
    cwd = os.getcwd()
    if cwd in sys.path:
        sys.path.remove(cwd)
    if '' in sys.path:
        sys.path.remove('')

    duri = cobra.swapCobraObject(uri, 'DcodeServer')
    cobra.dcode.addDcodeUri(duri)

    server = cobra.CobraProxy(uri, msgpack=True)
    scheme, host, port, name, urlparams = cobra.chopCobraUri(uri)

    module = importlib.import_module(name)

    if hasattr(module, 'remotemain'):
        module.remotemain(server)
    else:
        module.main()


def runRemoteApp(uri, join=True):
    p = multiprocessing.Process(target=getAndRunApp, args=(uri,))
    p.start()
    if join:
        p.join()


def execRemoteApp(uri):
    '''
    Exec a remoteapp without using multiprocessing (may be needed if fork()
    causes the child to have an unacceptably dirty environment)
    '''
    subprocess.Popen([sys.executable, '-m', 'cobra.remoteapp', uri])


def main():
    runRemoteApp(sys.argv[1])


if __name__ == '__main__':
    sys.exit(main())



'''
A home for the page lookup construct.  Basically it is a
python object which implements a similar lookup mechanism
to the i386 page table lookups...
'''
import collections
import envi.exc as e_exc

# FIXME move functions in here too so there is procedural "speed" way
# and objecty pythonic way...
def pagedict():
    return collections.defaultdict( lambda: [None] * 0xffff )

class PageLookup:
    '''
    An object capable of rapid lookups across a sparse address
    space which will also NOT eat *all* the RAMS like a straight
    dictionary full of millions of entries would.
    '''

    def __init__(self):
        self._page_dict = pagedict()

    def getPageLookup(self, va):
        page = self._page_dict.get( va >> 16 )
        if page is None:
            return None
        return page[ va & 0xffff ]

    def setPageLookup(self, va, size, obj):
        vamax = va+size

        p = self._page_dict
        # Super ugly, *very* fast speed hack
        [ p[ va >> 16 ].__setitem__( va & 0xffff, obj ) for va in range(va, vamax) ]

    # __getitem__
    # __getslice__
    # __setslice__

class MapLookup:

    '''
    A specialized lookup object for large densely populated ranges
    which are layed out in a sparse field space themselves...
    '''

    def __init__(self):
        self._maps_list = []

    def initMapLookup(self, va, size, obj=None):
        marray = [obj] * size
        # FIXME optimize by size!
        self._maps_list.append((va, va+size, marray))

    def setMapLookup(self, va, size, obj):
        for mva, mvamax, marray in self._maps_list:
            if va >= mva and va < mvamax:
                off = va - mva
                marray[off:off+size] = [obj] * size
                return
        raise e_exc.MapNotFoundException(va)

    def getMapLookup(self, va):
        for mva, mvamax, marray in self._maps_list:
            if va >= mva and va < mvamax:
                return marray[ va - mva ]
        return None

    def delMapLookup(self, va):
        for midx in range(len(self._maps_list)):
            mva, mvamax, marray = self._maps_list[midx]
            if va >= mva and va < mvamax:
                return self._maps_list.pop(midx)

        raise e_exc.MapNotFoundException(va=va)


    def __getslice__(self, start, end):
        raise NotImplementedError("__getslice__ on MapLookup needs implementing")



import sys
import logging
import argparse

import envi
import envi.memory as e_mem
import envi.archs.i386 as e_i386
import envi.archs.i386.opconst as e_i386const

import vtrace
import vtrace.exc as v_exc
import vtrace.util as vutil
import vtrace.snapshot as v_snapshot
import vtrace.platforms.base as v_base


logger = logging.getLogger(__name__)

undefs = {
    'amd64': {
        ('or', 'eflags'): e_i386.EFLAGS_AF,
        ('imul', 'eflags'): e_i386.EFLAGS_AF | e_i386.EFLAGS_ZF,
        ('shr', 'eflags'): e_i386.EFLAGS_AF,
        ('shl', 'eflags'): e_i386.EFLAGS_AF,
    },
    'i386': {
        ('or', 'eflags'): e_i386.EFLAGS_AF,
        ('imul', 'eflags'): e_i386.EFLAGS_AF,
        ('shr', 'eflags'): e_i386.EFLAGS_AF,
        ('shl', 'eflags'): e_i386.EFLAGS_AF,
    }
}

skip_mem_opcodes_by_arch = {
        'i386': (e_i386const.INS_NOP, e_i386const.INS_LEA),
        'amd64': (e_i386const.INS_NOP, e_i386const.INS_LEA),
        }

resync_reg_opcodes_by_arch = {
        'i386': (e_i386const.INS_RDTSC, e_i386const.INS_SYSCALL),
        'amd64': (e_i386const.INS_RDTSC, e_i386const.INS_SYSCALL),
        }


###### LockStepper and LockStepMonitor
class LockStepMonitor:
    '''
    This is an abstract class intended to be subclassed for different purposes

    Each callback should return True or False for whether the LockStepper
    should continue or fail
    '''
    def _cb_opcode_pre(self, lstep):
        '''
        Called before *each* instruction is executed.
        This (and _post) are outliers, where nothing actually went wrong.

        These callbacks are for use of analysis context only.
        '''
        return True

    def _cb_opcode_post(self, lstep):
        '''
        Called after *each* instruction is executed.
        This (and _pre) are outliers, where nothing actually went wrong.

        These callbacks are for use of analysis context only.
        '''
        return True

    def _cb_branch_failure(self, lstep, tracepc, emupc):
        '''
        Called when the Emulator and Tracer think the ProgramCounter should
        be different things.
        '''
        return False

    def _cb_decode_failure(self, lstep, traceop, emuop):
        '''
        Called when the Emulator and Tracer decode the next instruction should
        be different things.
        '''
        return False

    def _cb_register_failure(self, lstep, exc):
        '''
        Called when the Register Context differs between the Emu and Tracer
        '''
        return False

    def _cb_memory_failure(self, lstep, exc):
        '''
        Called when Memory differences are detected between the Emu and Tracer
        '''
        return False

    def _cb_trace_exc(self, lstep, exc):
        '''
        Called when Tracer.stepi() throws exceptions
        '''
        return False

    def _cb_emu_exc(self, lstep, exc):
        '''
        Called when Emu.stepi() throws exceptions
        '''
        return False

    def _cb_unknown_exception(self, lstep, exc):
        '''
        Called when we haven't prepared for this kind of exception!
        '''
        return False

    def _cb_segv(self):
        '''
        Called when a SegmentationViolation is thrown
        '''
        return False


class InteractiveLSMon(LockStepMonitor):
    '''
    This is a LockStepMonitor intended for Interactive LockStep Emulation

    It will prompt after certain failures, allowing the user to inspect issues
    and fix them with context.  After Register differences, user can type "s" 
    to resync the registers from the tdebug race.
    '''
    def __init__(self, arch):
        self.count = 0
        self.skip_mem_opcodes = skip_mem_opcodes_by_arch.get(arch)

    def _cb_opcode_pre(self, lstep):
        self.count += 1
        print("\n%4d  Lockstep: 0x%.8x  %-20r" % (self.count, lstep.emu.getProgramCounter(), lstep.op1), end='')
        op = lstep.op1
        for oper in op.opers:
            if op.opcode in self.skip_mem_opcodes:
                continue
            print("(%r = %x)" % (oper.repr(op), oper.getOperValue(op, lstep.emu)), end='')

        return True

    def _cb_opcode_post(self, lstep):
        print("(flags: 0x%x)" % lstep.trace.getStatusRegister(), end='')
        print("(flags: 0x%x)" % lstep.emu.getStatusRegister(), end='')
        return True

    def _cb_register_failure(self, lstep, exc):
        inp = input('"s" to reSync registers from trace to emu')
        if inp.startswith('s'):
            lstep.syncRegsFromTrace()
        return True

    def _cb_memory_failure(self, lstep, exc):
        inp = input('Memory Failure: "Enter" to continue')
        return True

    def _cb_segv(self):
        inp = input('Segmentation Violation: "Enter" to continue')
        return True


class LockStepper:
    '''
    LockStepper is used for comparing execution of an emulator to execution of 
    a debug trace.  An emulator copy is created of the debug trace, where 
    memory and registers are duplicated.  
    '''
    def __init__(self, trace, cbhandler=None):

        self.trace = trace
        if cbhandler:
            self.cbhandler = cbhandler
        else:
            self.cbhandler = LockStepMonitor()

        self.archname = trace.getMeta('Architecture')
        self.arch = envi.getArchModule(self.archname)

        self.emu = self.arch.getEmulator()
        self.memcache = e_mem.MemoryCache(self.trace)

        plat = trace.getMeta('Platform')
        self.resync_reg_opcodes = resync_reg_opcodes_by_arch.get(self.archname, ())

        # gotta setup fs at least...
        if plat == 'windows' and self.archname in ('i386', 'amd64'):
            # so 32 and 64 bit are flipped. In x86-land FS:[0] points to the TIB. On x64, it's GS:[0] that points to the
            # TEB
            tid = trace.getCurrentThread()
            tinfo = trace.getThreads().get(tid)
            if self.archname == 'i386':
                self.emu.setSegmentInfo(e_i386.SEG_FS, tinfo, 0xffffffff)
            elif self.archname == 'amd64':
                self.emu.setSegmentInfo(e_i386.SEG_GS, tinfo, 0xffffffff)

        # Monkey patch the emulator's read methods
        self.emu.readMemory = self.memcache.readMemory
        self.emu.writeMemory = self.memcache.writeMemory
        self.emu.getMemoryMap = self.memcache.getMemoryMap
        self.emu.getMemoryMaps = self.memcache.getMemoryMaps
        self.emu.parseOpcode = parseOpcode.__get__(self.emu)

        self.syncRegsFromTrace()

    def syncRegsFromTrace(self):
        '''
        Syncronizes the Register Context from the Trace to the Emulator
        '''
        regctx = self.trace.getRegisterContext()
        self.emu.setRegisterSnap(regctx.getRegisterSnap())

    def syncMemFromTrace(self):
        '''
        Syncronizes the MemoryMaps from the Trace to the Emulator
        (in reality, it clears the differences tracked)
        '''
        self.memcache.pagedirty = {}
        self.memcache.pagecache = {}

    def stepi(self, count=1):
        '''
        Execute/Emulate one instruction and compare memory/registers

        If issues are discovered, and if a callback handler has been registered,
        appropriate callbacks are triggered on that cbhandler.
        
        For example:
        cbhandler._cb_branch_failure
        cbhandler._cb_decode_failure
        cbhandler._cb_opcode_pre
        cbhandler._cb_register_failure
        cbhandler._cb_memory_failure
        cbhandler._cb_segv
        cbhandler._cb_unknown_exception
        '''
        cont = True
        for i in range(count):
            if not cont:
                break

            try:
                # get and compare Program Counter for both
                emupc = self.emu.getProgramCounter()
                tracepc = self.trace.getProgramCounter()
                if emupc != tracepc:
                    if not self.cbhandler._cb_branch_failure(self, tracepc, emupc):
                        break

                self.op1 = self.emu.parseOpcode(emupc)
                self.op2 = self.trace.parseOpcode(tracepc)
                if self.op1 != self.op2:
                    if not self.cbhandler._cb_decode_failure(self, self.op1, self.op2):
                        break

                if not self.cbhandler._cb_opcode_pre(self):
                    break

                try:
                    self.trace.stepi()

                except Exception as e:
                    raise Exception('Trace Exception: %s on %s' % (e, repr(self.op2)))

                try:
                    self.emu.stepi()

                except Exception as e:
                    raise Exception('Emu Exception: %s on %s' % (e, repr(self.op1)))

                self.cmpregs()
                self.cmppages()

            except v_exc.RegisterException as exc:
                logger.warning("    \tError: %s: %s" % (repr(self.op1), exc))
                cont = self.cbhandler._cb_register_failure(self, exc)

            except v_exc.MemoryException as exc:
                logger.warning("    \tError: %s: %s" % (repr(self.op1), exc))
                cont = self.cbhandler._cb_memory_failure(self, exc)

            except envi.SegmentationViolation as exc:
                logger.warning("    \tError: %s: %s" % (repr(self.op1), exc))
                cont = self.cbhandler._cb_segv(self)

            except Exception as exc:
                logger.warning("\t\tLockstep Error: %s" % exc, exc_info=1)
                if self.cbhandler:
                    cont = self.cbhandler._cb_unknown_exception(self, exc)

            finally:
                cont = cont and self.cbhandler._cb_opcode_post(self)

        logger.warning("done.")

    def cmpregs(self):
        if self.op1.opcode in self.resync_reg_opcodes:
            logger.info("Resynching Registers: %r" % self.op1)
            self.syncRegsFromTrace()
            return

        badregs = []
        emuregs = self.emu.getRegisters()
        traceregs = self.trace.getRegisters()
        for regname, regval in emuregs.items():
            traceval = traceregs.get(regname)
            mask = undefs.get(self.archname).get((self.op1.mnem, regname))
            if mask is not None:
                self.emu.setRegisterByName(regname, traceval)
                regval &= ~mask
                traceval &= ~mask

            if traceval != regval:
                badregs.append("%s (trace: 0x%.8x) (emulated: 0x%.8x)" % (regname, traceval, regval))
        
        if badregs:
            raise v_exc.RegisterException('LockStep: emu 0x%.8x %s %x | trace 0x%.8x %s: %r ' % (self.op1.va, repr(self.op1), self.op1.opcode, self.op2.va, repr(self.op2), '    '.join(badregs)), emuregs, traceregs)

    def cmppages(self):

        for va, bytez in self.memcache.getDirtyPages():
            tbytez = self.trace.readMemory(va, len(bytez))
            diffs = e_mem.memdiff(bytez, tbytez)
            if diffs:
                diffstr = ','.join(['0x%.8x: %d' % (va+offset, size) for offset, size in diffs])
                raise v_exc.MemoryException('LockStep: emu 0x%.8x %s | trace 0x%.8x %s | DIFFS: %s' % (self.op1.va, repr(self.op1), self.op2.va, repr(self.op2), diffstr))
        self.memcache.clearDirtyPages()

def parseOpcode(self, va, arch=envi.ARCH_DEFAULT):
    '''
    Monkey patching the emulator's actual getByteDef because the emu._map_defs isn't
    populated and populating it via just ripping through all the actual memory maps
    leads to a bunch of partial read errors that I don't feel like dealing with right now.
    '''
    byts = self.readMemory(va, 16)
    return self.imem_archs[(arch & envi.ARCH_MASK) >> 16].archParseOpcode(byts, 0, va)


###### TraceEmulator fusion magic
class TraceEmulator(vtrace.Trace, v_base.TracerBase):
    """
    Wrap an arbitrary emulator in a Tracer compatible API.
    """
    def __init__(self, emu):
        self.emu = emu
        archname = emu.vw.getMeta('Architecture')
        vtrace.Trace.__init__(self, archname)
        v_base.TracerBase.__init__(self)

        # Fake out being attached
        self.attached = True
        self.pid = 0x56

        self.setRegisterInfo(emu.getRegisterInfo())

    def getPointerSize(self):
        return self.emu.getPointerSize()

    def platformStepi(self):
        self.emu.stepi()

    def platformWait(self):
        # We only support single step events now
        return True

    def platformGetRegCtx(self, threadid):
        return self.emu

    def platformSetRegCtx(self, threadid, ctx):
        self.setRegisterSnap(ctx.getRegisterSnap())

    def platformProcessEvent(self, event):
        self.fireNotifiers(vtrace.NOTIFY_STEP)

    def platformReadMemory(self, va, size):
        return self.emu.readMemory(va, size)

    def platformWriteMemory(self, va, bytes):
        return self.emu.writeMemory(va, bytes)

    def platformGetMaps(self):
        return self.emu.getMemoryMaps()

    def platformGetThreads(self):
        return {1: 0xffff0000}

    def platformGetFds(self):
        return []  # FIXME perhaps tie this into magic?

    def archGetRegCtx(self):
        return self.emu

    def archGetRegisterGroups(self):
        return self.arch.archGetRegisterGroups()

    def pointerString(self, val):
        return self.arch.pointerString(val)

    def getStackTrace(self):
        # FIXME i386...
        return [(self.emu.getProgramCounter(), 0), (0, 0)]

    def platformDetach(self):
        pass

    def parseOpcode(self, va, arch=envi.ARCH_DEFAULT):
        return self.emu.parseOpcode(va, arch)


####### Command Line Support
def setup():
    ap = argparse.ArgumentParser('lockstep')
    ap.add_argument('--pid', type=int, help='PID of process to attach to (remember to have ptrace permissions)')
    ap.add_argument('--expr', type=str, help='Expression of an address to break the process on')
    ap.add_argument('--save', type=str, help='Save vtrace snapshot to the provided file')
    ap.add_argument('--args', type=str, help='Instead of attaching to a process, run this process with the given args')
    return ap


def main(argv):
    opts = setup().parse_args(argv)
    t = vtrace.getTrace()
    if opts.args:
        t.execute(opts.args)
    else:
        t.attach(opts.pid)

    if opts.expr:
        symaddr = t.parseExpression(opts.expr)
        t.addBreakpoint(vtrace.Breakpoint(symaddr))
        while t.getProgramCounter() != symaddr:
            t.run()
    if opts.save:
        # You may open this file in vdb to follow along
        snap = v_snapshot.takeSnapshot(t)
        snap.saveToFile(opts.save)

    arch = t.getArchName()
    lsmon = InteractiveLSMon(arch)
    lsr = LockStepper(t, lsmon)


    lsr.stepi(0xffffffff)


if __name__ == "__main__":
    # Copy this file out to the vtrace dir for testing and run as main
    sys.exit(main(sys.argv[1:]))



import vivisect.symboliks.emulator as vsym_emu


class MockEmulator(vsym_emu.SymbolikEmulator):
    __width__ = 4

    def __init__(self, vw):
        # super(TestEmulator, self).__init__(vw)
        vsym_emu.SymbolikEmulator.__init__(self, vw)



import sys
import struct
import argparse

from io import BytesIO
from itertools import cycle

import PE


MAX_OFFSET_PE_AFTER_MZ = 0x200


def xorbytes(data, key):
    return bytes([(x ^ y) for (x, y) in zip(data, cycle(key))])


def xorstatic(data, i):
    return bytes([(c ^ i) for c in data])


mz_xor = [(xorstatic(b'MZ', i), xorstatic(b'PE', i), i) for i in range(256)]


def carve(pbytes, offset=0):
    '''
    Return a list of (offset, size, xor) tuples of embedded PEs
    '''
    pblen = len(pbytes)
    todo = [(pbytes.find(mzx, offset), mzx, pex, i) for mzx, pex, i in mz_xor]
    todo = [(off, mzx, pex, i) for (off, mzx, pex, i) in todo if off != -1]

    while len(todo):

        off, mzx, pex, i = todo.pop()

        # The MZ header has one field we will check
        # e_lfanew is at 0x3c
        e_lfanew = off + 0x3c
        if pblen < (e_lfanew + 4):
            continue

        newoff = struct.unpack('<I', xorstatic(pbytes[e_lfanew:e_lfanew + 4], i))[0]

        # DEV: find the next possible slice *first* just in case this one fails
        nextres = pbytes.find(mzx, off+1)
        if nextres != -1:
            todo.append((nextres, mzx, pex, i))

        # PE header should occur soon after MZ
        if newoff > MAX_OFFSET_PE_AFTER_MZ:
            continue

        peoff = off + newoff
        if pblen < (peoff + 2):
            continue

        if pbytes[peoff:peoff + 2] == pex:
            yield (off, i)

class CarvedPE(PE.PE):

    def __init__(self, fbytes, offset, xkey):
        self.carved_offset = offset
        self.fbytes = fbytes
        self.xorkey = xkey
        PE.PE.__init__(self, BytesIO())
        # ensure sections can be parsed
        self.getSections()

    def readAtOffset(self, offset, size):
        offset += self.carved_offset
        return xorbytes(self.fbytes[offset:offset+size], self.xorkey)

    def getFileSize(self):
        ret = 0
        for sec in self.sections:
            ret = max(ret, sec.PointerToRawData + sec.SizeOfRawData)
        return ret


def setup():
    desc = 'Output info about PEs embedded inside another PE'
    ap = argparse.ArgumentParser('PE.carve', description=desc)
    ap.add_argument('file', help='Path to PE file')
    return ap


def main(argv):
    opts = setup().parse_args(argv)
    with open(opts.file, 'rb') as fd:
        fbytes = fd.read()
        for offset, i in carve(fbytes):
            print('OFFSET: %d (xor: %d)' % (offset, i))
            p = CarvedPE(fbytes, offset, chr(i))
            print('SIZE: %d' % p.getFileSize())


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import vstruct
from vstruct.primitives import *

class BITMAPINFOHEADER(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.biSize             = v_uint32()
        self.biWidth            = v_int32()
        self.biHeight           = v_int32()
        self.biPlanes           = v_uint16()
        self.biBitCount         = v_uint16()
        self.biCompression      = v_uint32()
        self.biSizeImage        = v_uint32()
        self.biXPelsPerMeter    = v_int32()
        self.biYPelsPerMeter    = v_int32()
        self.biClrUser          = v_uint32()
        self.biClrImportant     = v_uint32()






# APIs for Windows 32-bit MSVC 2005 runtime library (msvcr80).
# Format:  rettype, retname, callconv, exactname, arglist(type, name)
#          arglist type is one of ['int', 'void *']
#          arglist name is one of [None, 'funcptr', 'obj', 'ptr']
#
# The subset of functions defined below accounts for 80% of observed calls.

api_defs = {
    'msvcr80.??2@yapaxi@z':( 'int', None, 'cdecl', 'msvcr80.??2@YAPAXI@Z', (('int', None),) ),
    'msvcr80.??_u@yapaxi@z':( 'int', None, 'cdecl', 'msvcr80.??_U@YAPAXI@Z', (('int', None),) ),
    'msvcr80.??3@yaxpax@z':( 'void', None, 'cdecl', 'msvcr80.??3@YAXPAX@Z', (('void *', 'ptr'),) ),
    'msvcr80.??_v@yaxpax@z':( 'void', None, 'cdecl', 'msvcr80.??_V@YAXPAX@Z', (('void *', 'ptr'),) ),

    'msvcr80._cxxthrowexception':( 'void', None, 'stdcall', 'msvcr80._CxxThrowException', (('void *', 'ptr'), ('int', None)) ),
    'msvcr80.__iob_func':( 'int', None, 'cdecl', 'msvcr80.__iob_func', () ),
    'msvcr80._invalid_parameter_noinfo':( 'void', None, 'cdecl', 'msvcr80._invalid_parameter_noinfo', () ),
    'msvcr80._encode_pointer':( 'int', None, 'cdecl', 'msvcr80._encode_pointer', (('int', None),) ),
    'msvcr80._errno':( 'int', None, 'cdecl', 'msvcr80._errno', () ),

    'msvcr80._exit':( 'int', None, 'cdecl', 'msvcr80._exit', (('int', None),) ),
    'msvcr80.abort':( 'int', None, 'cdecl', 'msvcr80.abort', () ),
    'msvcr80.exit':( 'int', None, 'cdecl', 'msvcr80.exit', (('int', None),) ),
    'msvcr80.quick_exit':( 'int', None, 'cdecl', 'msvcr80.quick_exit', (('int', None),) ),

    'msvcr80.calloc':( 'int', None, 'cdecl', 'msvcr80.calloc', (('int', None), ('int', None)) ),
    'msvcr80.malloc':( 'int', None, 'cdecl', 'msvcr80.malloc', (('int', None),) ),
    'msvcr80.free':( 'void', None, 'cdecl', 'msvcr80.free', (('void *', 'ptr'),) ),

    'msvcr80.memcmp':( 'int', None, 'cdecl', 'msvcr80.memcmp', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr80.memcpy':( 'int', None, 'cdecl', 'msvcr80.memcpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr80.memmove':( 'int', None, 'cdecl', 'msvcr80.memmove', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr80.memmove_s':( 'int', None, 'cdecl', 'msvcr80.memmove_s', (('void *', 'ptr'), ('int', None), ('void *', 'ptr'), ('int', None)) ),
    'msvcr80.memset':( 'int', None, 'cdecl', 'msvcr80.memset', (('void *', 'ptr'), ('int', None), ('int', None)) ),

    'msvcr80._snprintf':( 'int', None, 'cdecl', 'msvcr80._snprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcr80._swprintf':( 'int', None, 'cdecl', 'msvcr80._snprintf', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcr80.fprintf':( 'int', None, 'cdecl', 'msvcr80.fprintf', (('void *', 'ptr'), ('int', None)) ),
    'msvcr80.sprintf':( 'int', None, 'cdecl', 'msvcr80.sprintf', (('int', None), ('int', None)) ),
    'msvcr80.printf':( 'int', None, 'cdecl', 'msvcr80.printf', (('int', None),) ),

    'msvcr80._stricmp':( 'int', None, 'cdecl', 'msvcr80._stricmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr80.strchr':( 'int', None, 'cdecl', 'msvcr80.strchr', (('void *', 'ptr'), ('int', None)) ),
    'msvcr80.strcmp':( 'int', None, 'cdecl', 'msvcr80.strcmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr80.strcpy':( 'int', None, 'cdecl', 'msvcr80.strcpy', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr80.strlen':( 'int', None, 'cdecl', 'msvcr80.strlen', (('void *', 'ptr'),) ),
    'msvcr80.strncmp':( 'int', None, 'cdecl', 'msvcr80.strncmp', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr80.strncpy':( 'int', None, 'cdecl', 'msvcr80.strncpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr80.strstr':( 'int', None, 'cdecl', 'msvcr80.strstr', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr80._wcsicmp':( 'int', None, 'cdecl', 'msvcr80._wcsicmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr80.wcslen':( 'int', None, 'cdecl', 'msvcr80.wcslen', (('void *', 'ptr'),) ),
    'msvcr80.wcsrchr':( 'int', None, 'cdecl', 'msvcr80.wcsrchr', (('void *', 'ptr'), ('int', None)) ),

    'msvcr80._time64':( 'int', None, 'cdecl', 'msvcr80._time64', (('void *', 'ptr'),) ),
    'msvcr80.atoi':( 'int', None, 'cdecl', 'msvcr80.atoi', (('void *', 'ptr'),) ),
    'msvcr80.getenv':( 'int', None, 'cdecl', 'msvcr80.getenv', (('void *', 'ptr'),) ),
    'msvcr80.fclose':( 'int', None, 'cdecl', 'msvcr80.fclose', (('void *', 'ptr'),) ),
    'msvcr80.bsearch':( 'int', None, 'cdecl', 'msvcr80.bsearch', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None), ('int', None), ('void *', 'funcptr')) )
}



#!/usr/bin/env python
import os
import sys
import time
import logging
import argparse
import cProfile
import importlib.util

import envi
import envi.exc as e_exc
import envi.common as e_common
import envi.threads as e_threads

import vivisect.cli as viv_cli
import vivisect.parsers as viv_parsers


logger = logging.getLogger('vivisect')


def main():
    parser = argparse.ArgumentParser(prog='vivbin', usage='%(prog)s [options] <workspace|binaries...>')
    parser.add_argument('-M', '--module', dest='modname', default=None, action='store',
                        help='run the file listed as an analysis module in non-gui mode and exit')
    parser.add_argument('-A', '--skip-analysis', dest='doanalyze', default=True, action='store_false',
                        help='Do *not* do an initial auto-analysis pass')
    parser.add_argument('-B', '--bulk', dest='bulk', default=False, action='store_true',
                        help='Do *not* start the gui, just load, analyze and save')
    parser.add_argument('-C', '--cprofile', dest='cprof', default=False, action='store_true',
                        help='Output vivisect performace profiling (cProfile) info')
    parser.add_argument('-E', '--entrypoint', dest='entrypoints', default=[], action='append',
                        help='Add Entry Point for bulk analysis (can have multiple "-E <addr>" args')
    parser.add_argument('-O', '--option', dest='option', default=None, action='append',
                        help='<secname>.<optname>=<optval> (optval must be json syntax)')
    parser.add_argument('-p', '--parser', dest='parsemod', default=None, action='store',
                        help='Manually specify the parser module (pe/elf/blob/...)')
    parser.add_argument('-s', '--storage', dest='storage_name', default=None, action='store',
                        help='Specify a storage module by name')
    parser.add_argument('-S', '--server', dest='server_mode', default=False, action='store_true',
                        help='Run Vivisect Server.  Last argument should be the VivWorkspaces directory.')
    parser.add_argument('-P', '--port', dest='server_port', default=None, action='store',
                        help='Port to run Vivisect Server on (only meaningful with --server)')
    parser.add_argument('-v', '--verbose', dest='verbose', default=False, action='count',
                        help='Enable verbose mode (multiples matter: -vvvv)')
    parser.add_argument('-V', '--version', dest='version', default=None, action='store',
                        help='Add file version (if available) to save file name')
    parser.add_argument('-c', '--config', dest='config', default=None,
                        help='Path to a directory to use for config data')
    parser.add_argument('-a', '--autosave', dest='autosave', default=False, action='store_true',
                        help='Autosave configuration data')
    parser.add_argument('-o', '--outfile', default=None,
                        help='Name of VivWorkspace file to create (useful for loading multiple binaries into one workspace)')
    parser.add_argument('-m', '--archmaturity', default=False, action='store_true',
                        help="List Architectures and their version/maturity")

    parser.add_argument('file', nargs='*')
    args = parser.parse_args()

    vw = viv_cli.VivCli(confdir=args.config, autosave=args.autosave)

    # setup logging
    vw.verbose = min(args.verbose, len(e_common.LOG_LEVELS)-1)
    level = e_common.LOG_LEVELS[vw.verbose]
    e_common.initLogging(logger, level=level)

    if args.server_mode:
        import vivisect.remote.server as vr_server
        if len(args.file):
            file = args.file[-1]

        else:
            print("Server mode requires directory name as last argument")
            sys.exit(-1)

        if args.server_port is None:
            vr_server.runMainServer(dirname=file)
        else:
            server_port = int(args.server_port)
            vr_server.runMainServer(dirname=file, port=server_port)
        sys.exit(0)

    if args.outfile:
        vw.setMeta('StorageName', args.outfile)


    if args.archmaturity:
        # This is it.  Nothing happens beyond this point :)

        # make sure we are at least at debug level INFO
        if level > logging.INFO:
            logger.setLevel(logging.INFO)

        logger.info("Supported Architectures, Version, and Features")
        logger.info("==============================================")
        for arch, m in envi.arch_defs.items():
            logger.info("%20s  ver: v%d.%d.%d  aliases: %s", m['name'], *m['version'], m.get('aliases'))
            logger.debug("\t\t\tdisasm: %5s   emu: %5s   symboliks: %5s   unittests: %5s\n", 
                    m['has_disasm'],
                    m['has_emu'],
                    m['has_symboliks'],
                    m['has_unittests'])
        sys.exit(-1)

    # do things
    if args.option is not None:
        for option in args.option:
            if option in ('-h', '?'):
                logger.critical(vw.config.reprConfigPaths())
                sys.exit(-1)

            try:
                vw.config.parseConfigOption(option)
            except e_exc.ConfigNoAssignment as e:
                logger.critical(vw.config.reprConfigPaths() + "\n")
                logger.critical(e)
                logger.critical("syntax: \t-O <secname>.<optname>=<optval> (optval must be json syntax)")
                sys.exit(-1)

            except Exception as e:
                logger.critical(vw.config.reprConfigPaths())
                logger.critical("With entry: %s", option)
                logger.critical(e)
                sys.exit(-1)

    if args.storage_name is not None:
        vw.setMeta("StorageModule", args.storage_name)

    # If we're not gonna load files, no analyze
    if args.file is None:
        args.doanalyze = False

    # Load in any additional files...
    needanalyze = False
    if args.file is not None:
        for fname in args.file:
            if args.parsemod is None:
                args.parsemod = viv_parsers.guessFormatFilename(fname)

            start = time.time()
            if args.parsemod == 'viv':
                vw.loadWorkspace(fname)
            else:
                needanalyze = True
                vw.loadFromFile(fname, fmtname=args.parsemod)

            end = time.time()
            logger.info('Loaded (%.4f sec) %s', (end - start), fname)

    if args.bulk:
        for entryva in args.entrypoints:
            try:
                vw.vprint("Adding Entry Point: %s: " % entryva)
                eva = int(entryva, 0)
                vw.setVaSetRow('EntryPoints', (eva,))
            except Exception as e:
                vw.vprint("Failure: %r" % e)

        if args.doanalyze:
            if args.cprof:
                cProfile.run("vw.analyze()")
            else:
                start = time.time()
                vw.analyze()
                end = time.time()
                logger.debug("ANALYSIS TIME: %s", (end-start))

        if args.modname is not None:
            modpath = os.path.abspath(args.modname)
            spec = importlib.util.spec_from_file_location('custom_analysis', modpath)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
            module.analyze(vw)

        logger.info('stats: %r', vw.getStats())
        logger.info("Saving workspace: %s", vw.getMeta('StorageName'))
        vw.saveWorkspace()

    else:

        import vivisect.qt.main as viv_qt_main

        # If we are interactive, lets turn on extended output...
        if args.doanalyze and needanalyze:
            e_threads.firethread(vw.analyze)()

        viv_qt_main.main(vw)


if __name__ == '__main__':
    main()



'''
Home of the newer, cleaner iterators (yield based generators...)
'''
import envi
import vivisect


def iterOps(vw, va):
    '''
    Iterate over the virtual addresses of all the code blocks from the
    specified function downward...

    Example: for va in fscope.iterOps(vw, funcaddr):
    '''
    fva = vw.getFunction(va)
    fdone = {}
    todo = [fva, ]
    while len(todo):
        fva = todo.pop()
        if fdone.get(fva, False):
            continue
        fdone[fva] = True

        for cbva, cbsize, fva in vw.getFunctionBlocks(fva):

            cbend = cbva + cbsize
            while cbva < cbend:

                yield cbva

                # Go through the locations in this code block
                lva, lsize, ltype, linfo = vw.getLocation(cbva)
                for xrfrom, xrto, rtype, rflags in vw.getXrefsFrom(lva):
                    if rtype != vivisect.REF_CODE:
                        continue

                    # Skip non-procedural branches
                    if not rflags & envi.BR_PROC:
                        continue

                    # Skip dereference calls
                    if rflags & envi.BR_DEREF:
                        continue

                    # Procedural code branches w/o deref should always
                    # target other functions.
                    if not vw.isFunction(xrto):
                        continue

                    # Check if we've done this function before
                    if fdone.get(xrto):
                        continue

                    todo.append(xrto)

                # round and round!
                cbva += lsize


def getImportCalls(vw, va):
    '''
    Get all the import calls which happen from the given function
    and down....

    Example: for callva,impname in getImportCalls(vw, fva)
    '''

    ret = []
    for lva in iterOps(vw, va):

        for xrfrom, xrto, rtype, rflags in vw.getXrefsFrom(lva):
            if rtype != vivisect.REF_CODE:
                continue

            # Skip non-procedural branches
            if not rflags & envi.BR_PROC:
                continue

            # This may be an actual import call!
            if rflags & envi.BR_DEREF:
                loc = vw.getLocation(xrto)
                if loc is not None:
                    drva, drsize, drtype, drinfo = loc
                    if drtype == vivisect.LOC_IMPORT:
                        ret.append((lva, drinfo))
    return ret


def getStringRefs(vw, va):
    '''
    Get all of the immedates which point to string or unicode types
    in the workspace for this function scope. Returns a list of
    (refva, strva, strval) tuples.

    Example: for instrva, strva, strrepr in getStringRefs(vw, funcva)
    '''
    ret = []
    for lva in iterOps(vw, va):
        for xrfrom, xrto, rtype, rflags in vw.getXrefsFrom(lva):
            if rtype != vivisect.REF_PTR:
                continue

            ploc = vw.getLocation(xrto)
            if ploc is None:
                continue

            plva, plsize, pltype, plinfo = ploc
            if pltype == vivisect.LOC_STRING:
                r = vw.reprLocation(ploc)
                ret.append((lva, plva, r))
            elif pltype == vivisect.LOC_UNI:
                r = vw.reprLocation(ploc)
                ret.append((lva, plva, r))

    return ret



import unittest

from vivisect.const import *
from vivisect.symboliks.common import *
from vivisect.symboliks.expression import symexp

class TestSymbolikCache(unittest.TestCase):
    '''
    tests the reduction of asts consisting of add's and sub's if widths are
    the same.
    '''
    def test_symboliks_cache_solve_vars(self):
        s = symexp('x + 20')

        self.assertEqual(s.solve(vals={'x':10}), 30)
        self.assertEqual(s.solve(vals={'x':20}), 40)

        self.assertIsNone(s.cache.get('solve'))
        s.solve()
        self.assertIsNotNone(s.cache.get('solve'))

    def test_symboliks_cache_walktree(self):

        s = symexp('x + 30')

        solved1 = s.solve()

        self.assertIsNotNone(s.cache.get('solve'))

        def swapx(path,sym,ctx):
            if sym.symtype == SYMT_VAR and sym.name == 'x':
                return Var('y',4)

        s = s.walkTree(swapx)

        # check the partial cache clear
        self.assertIsNone(s.cache.get('solve'))
        self.assertIsNone(s.kids[0].cache.get('solve'))
        self.assertIsNotNone(s.kids[1].cache.get('solve'))

        solved2 = s.solve()
        self.assertEqual(str(s), '(y + 30)')

        self.assertNotEqual(solved1, solved2)

        # be *really* mean and manually slice a kid
        # then confirm that the cache still hits..
        s.kids[0] = Var('x', 4)
        self.assertEqual(s.solve(), solved2)




class VisGraphException(Exception):
    pass

class DuplicateNode(VisGraphException):
    def __init__(self, node):
        Exception.__init__(self, repr(node))
        self.node = node

class NodeNonExistant(VisGraphException):
    def __init__(self, nodeid):
        self.nodeid = nodeid
        Exception.__init__(self, 'Node %d does not exist!' % nodeid)

class EdgeNonExistant(VisGraphException):
    def __init__(self, edgeid):
        self.edgeid = edgeid
        Exception.__init__(self, 'Edge %d does not exist!' % edgeid)
